@@ -31,7 +31,8 @@ from bson.binary import (Binary, OLD_UUID_SUBTYPE,
                          JAVA_LEGACY, CSHARP_LEGACY,
                          UUIDLegacy)
 from bson.code import Code
-from bson.codec_options import CodecOptions, DEFAULT_CODEC_OPTIONS
+from bson.codec_options import (
+    CodecOptions, DEFAULT_CODEC_OPTIONS, _raw_document_class)
 from bson.dbref import DBRef
 from bson.errors import (InvalidBSON,
                          InvalidDocument,
@@ -98,10 +99,11 @@ def _get_int(data, position, dummy0, dummy1):
     return _UNPACK_INT(data[position:end])[0], end
 
 
-def _get_c_string(data, position):
+def _get_c_string(data, position, opts):
     """Decode a BSON 'C' string to python unicode string."""
     end = data.index(b"\x00", position)
-    return _utf_8_decode(data[position:end], None, True)[0], end + 1
+    return _utf_8_decode(data[position:end],
+                         opts.unicode_decode_error_handler, True)[0], end + 1
 
 
 def _get_float(data, position, dummy0, dummy1):
@@ -110,7 +112,7 @@ def _get_float(data, position, dummy0, dummy1):
     return _UNPACK_FLOAT(data[position:end])[0], end
 
 
-def _get_string(data, position, obj_end, dummy):
+def _get_string(data, position, obj_end, opts):
     """Decode a BSON string to python unicode string."""
     length = _UNPACK_INT(data[position:position + 4])[0]
     position += 4
@@ -119,7 +121,8 @@ def _get_string(data, position, obj_end, dummy):
     end = position + length - 1
     if data[end:end + 1] != b"\x00":
         raise InvalidBSON("invalid end of string")
-    return _utf_8_decode(data[position:end], None, True)[0], end + 1
+    return _utf_8_decode(data[position:end],
+                         opts.unicode_decode_error_handler, True)[0], end + 1
 
 
 def _get_object(data, position, obj_end, opts):
@@ -130,6 +133,10 @@ def _get_object(data, position, obj_end, opts):
         raise InvalidBSON("bad eoo")
     if end >= obj_end:
         raise InvalidBSON("invalid object length")
+    if _raw_document_class(opts.document_class):
+        return (opts.document_class(data[position:end + 1], opts),
+                position + obj_size)
+
     obj = _elements_to_dict(data, position + 4, end, opts)
 
     position += obj_size
@@ -145,6 +152,7 @@ def _get_array(data, position, obj_end, opts):
     end = position + size - 1
     if data[end:end + 1] != b"\x00":
         raise InvalidBSON("bad eoo")
+
     position += 4
     end -= 1
     result = []
@@ -215,11 +223,14 @@ def _get_date(data, position, dummy, opts):
     seconds = (millis - diff) / 1000
     micros = diff * 1000
     if opts.tz_aware:
-        return EPOCH_AWARE + datetime.timedelta(
-            seconds=seconds, microseconds=micros), end
+        dt = EPOCH_AWARE + datetime.timedelta(
+            seconds=seconds, microseconds=micros)
+        if opts.tzinfo:
+            dt = dt.astimezone(opts.tzinfo)
     else:
-        return EPOCH_NAIVE + datetime.timedelta(
-            seconds=seconds, microseconds=micros), end
+        dt = EPOCH_NAIVE + datetime.timedelta(
+            seconds=seconds, microseconds=micros)
+    return dt, end
 
 
 def _get_code(data, position, obj_end, opts):
@@ -235,10 +246,10 @@ def _get_code_w_scope(data, position, obj_end, opts):
     return Code(code, scope), position
 
 
-def _get_regex(data, position, dummy0, dummy1):
+def _get_regex(data, position, dummy0, opts):
     """Decode a BSON regex to bson.regex.Regex or a python pattern object."""
-    pattern, position = _get_c_string(data, position)
-    bson_flags, position = _get_c_string(data, position)
+    pattern, position = _get_c_string(data, position, opts)
+    bson_flags, position = _get_c_string(data, position, opts)
     bson_re = Regex(pattern, bson_flags)
     return bson_re, position
 
@@ -295,18 +306,25 @@ def _element_to_dict(data, position, obj_end, opts):
     """Decode a single key, value pair."""
     element_type = data[position:position + 1]
     position += 1
-    element_name, position = _get_c_string(data, position)
+    element_name, position = _get_c_string(data, position, opts)
     value, position = _ELEMENT_GETTER[element_type](data,
                                                     position, obj_end, opts)
     return element_name, value, position
+if _USE_C:
+    _element_to_dict = _cbson._element_to_dict
+
+
+def _iterate_elements(data, position, obj_end, opts):
+    end = obj_end - 1
+    while position < end:
+        (key, value, position) = _element_to_dict(data, position, obj_end, opts)
+        yield key, value
 
 
 def _elements_to_dict(data, position, obj_end, opts):
     """Decode a BSON document."""
     result = opts.document_class()
-    end = obj_end - 1
-    while position < end:
-        (key, value, position) = _element_to_dict(data, position, obj_end, opts)
+    for key, value in _iterate_elements(data, position, obj_end, opts):
         result[key] = value
     return result
 
@@ -322,6 +340,8 @@ def _bson_to_dict(data, opts):
     if data[obj_size - 1:obj_size] != b"\x00":
         raise InvalidBSON("bad eoo")
     try:
+        if _raw_document_class(opts.document_class):
+            return opts.document_class(data, opts)
         return _elements_to_dict(data, 4, obj_size - 1, opts)
     except InvalidBSON:
         raise
@@ -424,6 +444,8 @@ else:
 
 def _encode_mapping(name, value, check_keys, opts):
     """Encode a mapping type."""
+    if _raw_document_class(value):
+        return b'\x03' + name + value.raw
     data = b"".join([_element_to_bson(key, val, check_keys, opts)
                      for key, val in iteritems(value)])
     return b"\x03" + name + _PACK_INT(len(data) + 5) + data + b"\x00"
@@ -689,6 +711,8 @@ def _element_to_bson(key, value, check_keys, opts):
 
 def _dict_to_bson(doc, check_keys, opts, top_level=True):
     """Encode a document to BSON."""
+    if _raw_document_class(doc):
+        return doc.raw
     try:
         elements = []
         if top_level and "_id" in doc:
@@ -746,6 +770,7 @@ def decode_all(data, codec_options=DEFAULT_CODEC_OPTIONS):
     docs = []
     position = 0
     end = len(data) - 1
+    use_raw = _raw_document_class(codec_options.document_class)
     try:
         while position < end:
             obj_size = _UNPACK_INT(data[position:position + 4])[0]
@@ -754,10 +779,15 @@ def decode_all(data, codec_options=DEFAULT_CODEC_OPTIONS):
             obj_end = position + obj_size - 1
             if data[obj_end:position + obj_size] != b"\x00":
                 raise InvalidBSON("bad eoo")
-            docs.append(_elements_to_dict(data,
-                                          position + 4,
-                                          obj_end,
-                                          codec_options))
+            if use_raw:
+                docs.append(
+                    codec_options.document_class(
+                        data[position:obj_end + 1], codec_options))
+            else:
+                docs.append(_elements_to_dict(data,
+                                              position + 4,
+                                              obj_end,
+                                              codec_options))
             position += obj_size
         return docs
     except InvalidBSON:
