@@ -1,4 +1,4 @@
-# Copyright 2009-2014 MongoDB, Inc.
+# Copyright 2009-2015 MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -59,49 +59,27 @@ but it will be faster as there is less recursion.
 
 .. versionchanged:: 2.7
    Preserves order when rendering SON, Timestamp, Code, Binary, and DBRef
-   instances. (But not in Python 2.4.)
+   instances.
 
 .. versionchanged:: 2.3
    Added dumps and loads helpers to automatically handle conversion to and
    from json and supports :class:`~bson.binary.Binary` and
    :class:`~bson.code.Code`
-
-.. versionchanged:: 1.9
-   Handle :class:`uuid.UUID` instances, whenever possible.
-
-.. versionchanged:: 1.8
-   Handle timezone aware datetime instances on encode, decode to
-   timezone aware datetime instances.
-
-.. versionchanged:: 1.8
-   Added support for encoding/decoding :class:`~bson.max_key.MaxKey`
-   and :class:`~bson.min_key.MinKey`, and for encoding
-   :class:`~bson.timestamp.Timestamp`.
-
-.. versionchanged:: 1.2
-   Added support for encoding/decoding datetimes and regular expressions.
 """
 
 import base64
 import calendar
+import collections
 import datetime
+import json
 import re
-import time
+import uuid
 
-json_lib = True
-try:
-    import json
-except ImportError:
-    try:
-        import simplejson as json
-    except ImportError:
-        json_lib = False
-
-import bson
 from bson import EPOCH_AWARE, RE_TYPE, SON
 from bson.binary import Binary
 from bson.code import Code
 from bson.dbref import DBRef
+from bson.int64 import Int64
 from bson.max_key import MaxKey
 from bson.min_key import MinKey
 from bson.objectid import ObjectId
@@ -109,7 +87,7 @@ from bson.regex import Regex
 from bson.timestamp import Timestamp
 from bson.tz_util import utc
 
-from bson.py3compat import PY3, binary_type, string_types
+from bson.py3compat import PY3, iteritems, string_type, text_type
 
 
 _RE_OPT_TABLE = {
@@ -130,10 +108,8 @@ def dumps(obj, *args, **kwargs):
 
     .. versionchanged:: 2.7
        Preserves order when rendering SON, Timestamp, Code, Binary, and DBRef
-       instances. (But not in Python 2.4.)
+       instances.
     """
-    if not json_lib:
-        raise Exception("No json library available")
     return json.dumps(_json_convert(obj), *args, **kwargs)
 
 
@@ -141,20 +117,8 @@ def loads(s, *args, **kwargs):
     """Helper function that wraps :class:`json.loads`.
 
     Automatically passes the object_hook for BSON type conversion.
-
-    :Parameters:
-      - `compile_re` (optional): if ``False``, don't attempt to compile BSON
-        regular expressions into Python regular expressions. Return instances
-        of :class:`~bson.bsonregex.BSONRegex` instead.
-
-    .. versionchanged:: 2.7
-       Added ``compile_re`` option.
     """
-    if not json_lib:
-        raise Exception("No json library available")
-
-    compile_re = kwargs.pop('compile_re', True)
-    kwargs['object_hook'] = lambda dct: object_hook(dct, compile_re)
+    kwargs['object_hook'] = lambda dct: object_hook(dct)
     return json.loads(s, *args, **kwargs)
 
 
@@ -163,8 +127,8 @@ def _json_convert(obj):
     converted into json.
     """
     if hasattr(obj, 'iteritems') or hasattr(obj, 'items'):  # PY3 support
-        return SON(((k, _json_convert(v)) for k, v in obj.iteritems()))
-    elif hasattr(obj, '__iter__') and not isinstance(obj, string_types):
+        return SON(((k, _json_convert(v)) for k, v in iteritems(obj)))
+    elif hasattr(obj, '__iter__') and not isinstance(obj, (text_type, bytes)):
         return list((_json_convert(v) for v in obj))
     try:
         return default(obj)
@@ -172,7 +136,7 @@ def _json_convert(obj):
         return obj
 
 
-def object_hook(dct, compile_re=True):
+def object_hook(dct):
     if "$oid" in dct:
         return ObjectId(str(dct["$oid"]))
     if "$ref" in dct:
@@ -180,28 +144,29 @@ def object_hook(dct, compile_re=True):
     if "$date" in dct:
         dtm = dct["$date"]
         # mongoexport 2.6 and newer
-        if isinstance(dtm, basestring):
-            # datetime.datetime.strptime is new in python 2.5
-            naive = datetime.datetime(
-                *(time.strptime(dtm[:19], "%Y-%m-%dT%H:%M:%S")[0:6]))
-            # The %f format is new in python 2.6
-            micros = int(dtm[20:23]) * 1000
-            aware = naive.replace(microsecond=micros, tzinfo=utc)
+        if isinstance(dtm, string_type):
+            aware = datetime.datetime.strptime(
+                dtm[:23], "%Y-%m-%dT%H:%M:%S.%f").replace(tzinfo=utc)
             offset = dtm[23:]
-            if not offset:
-                # No offset, assume UTC.
+            if not offset or offset == 'Z':
+                # UTC
                 return aware
-            elif len(offset) == 5:
-                # Offset from mongoexport is in format (+|-)HHMM
-                secs = (int(offset[1:3]) * 3600 + int(offset[3:]) * 60)
+            else:
+                if len(offset) == 5:
+                    # Offset from mongoexport is in format (+|-)HHMM
+                    secs = (int(offset[1:3]) * 3600 + int(offset[3:]) * 60)
+                elif ':' in offset and len(offset) == 6:
+                    # RFC-3339 format (+|-)HH:MM
+                    hours, minutes = offset[1:].split(':')
+                    secs = (int(hours) * 3600 + int(minutes) * 60)
+                else:
+                    # Not RFC-3339 compliant or mongoexport output.
+                    raise ValueError("invalid format for offset")
                 if offset[0] == "-":
                     secs *= -1
                 return aware - datetime.timedelta(seconds=secs)
-            else:
-                # Some other tool created this, or mongoexport changed again?
-                raise ValueError("invalid format for offset")
         # mongoexport 2.6 and newer, time before the epoch (SERVER-15275)
-        elif isinstance(dtm, dict):
+        elif isinstance(dtm, collections.Mapping):
             secs = float(dtm["$numberLong"]) / 1000.0
         # mongoexport before 2.6
         else:
@@ -212,11 +177,7 @@ def object_hook(dct, compile_re=True):
         # PyMongo always adds $options but some other tools may not.
         for opt in dct.get("$options", ""):
             flags |= _RE_OPT_TABLE.get(opt, 0)
-
-        if compile_re:
-            return re.compile(dct["$regex"], flags)
-        else:
-            return Regex(dct["$regex"], flags)
+        return Regex(dct["$regex"], flags)
     if "$minKey" in dct:
         return MinKey()
     if "$maxKey" in dct:
@@ -230,14 +191,12 @@ def object_hook(dct, compile_re=True):
         return Binary(base64.b64decode(dct["$binary"].encode()), subtype)
     if "$code" in dct:
         return Code(dct["$code"], dct.get("$scope"))
-    if bson.has_uuid() and "$uuid" in dct:
-        return bson.uuid.UUID(dct["$uuid"])
+    if "$uuid" in dct:
+        return uuid.UUID(dct["$uuid"])
     if "$undefined" in dct:
         return None
     if "$numberLong" in dct:
-        # 2to3 will change this to int. PyMongo 3.0 supports
-        # a new type, Int64, to avoid round trip issues.
-        return long(dct["$numberLong"])
+        return Int64(dct["$numberLong"])
     if "$timestamp" in dct:
         tsp = dct["$timestamp"]
         return Timestamp(tsp["t"], tsp["i"])
@@ -246,13 +205,7 @@ def object_hook(dct, compile_re=True):
 
 def default(obj):
     # We preserve key order when rendering SON, DBRef, etc. as JSON by
-    # returning a SON for those types instead of a dict. This works with
-    # the "json" standard library in Python 2.6+ and with simplejson
-    # 2.1.0+ in Python 2.5+, because those libraries iterate the SON
-    # using PyIter_Next. Python 2.4 must use simplejson 2.0.9 or older,
-    # and those versions of simplejson use the lower-level PyDict_Next,
-    # which bypasses SON's order-preserving iteration, so we lose key
-    # order in Python 2.4.
+    # returning a SON for those types instead of a dict.
     if isinstance(obj, ObjectId):
         return {"$oid": str(obj)}
     if isinstance(obj, DBRef):
@@ -278,7 +231,7 @@ def default(obj):
             flags += "u"
         if obj.flags & re.VERBOSE:
             flags += "x"
-        if isinstance(obj.pattern, unicode):
+        if isinstance(obj.pattern, text_type):
             pattern = obj.pattern
         else:
             pattern = obj.pattern.decode('utf-8')
@@ -295,10 +248,10 @@ def default(obj):
         return SON([
             ('$binary', base64.b64encode(obj).decode()),
             ('$type', "%02x" % obj.subtype)])
-    if PY3 and isinstance(obj, binary_type):
+    if PY3 and isinstance(obj, bytes):
         return SON([
             ('$binary', base64.b64encode(obj).decode()),
             ('$type', "00")])
-    if bson.has_uuid() and isinstance(obj, bson.uuid.UUID):
+    if isinstance(obj, uuid.UUID):
         return {"$uuid": obj.hex}
     raise TypeError("%r is not JSON serializable" % obj)
