@@ -16,13 +16,17 @@
 """Functions and classes common to multiple pymongo modules."""
 
 import collections
+import warnings
 
 from bson.binary import (STANDARD, PYTHON_LEGACY,
                          JAVA_LEGACY, CSHARP_LEGACY)
 from bson.codec_options import CodecOptions
-from bson.py3compat import string_type, integer_types
+from bson.py3compat import string_type, integer_types, iteritems
+from bson.raw_bson import RawBSONDocument
 from pymongo.auth import MECHANISMS
 from pymongo.errors import ConfigurationError
+from pymongo.monitoring import _validate_event_listeners
+from pymongo.read_concern import ReadConcern
 from pymongo.read_preferences import (read_pref_mode_from_name,
                                       _ServerMode)
 from pymongo.ssl_support import validate_cert_reqs
@@ -71,6 +75,9 @@ LOCAL_THRESHOLD_MS = 15
 # exist.
 COMMAND_NOT_FOUND_CODES = (59, 13390, None)
 
+# Error codes to ignore if GridFS calls createIndex on a secondary
+UNAUTHORIZED_CODES = (13, 16547, 16548)
+
 
 def partition_node(node):
     """Split a host:port string into (host, int(port)) pair."""
@@ -338,22 +345,24 @@ def validate_auth_mechanism_properties(option, value):
     for opt in value.split(','):
         try:
             key, val = opt.split(':')
-            if key not in _MECHANISM_PROPS:
-                raise ValueError("%s is not a supported auth "
-                                 "mechanism property. Must be one of "
-                                 "%s." % (key, tuple(_MECHANISM_PROPS)))
-            props[key] = val
         except ValueError:
             raise ValueError("auth mechanism properties must be "
                              "key:value pairs like SERVICE_NAME:"
                              "mongodb, not %s." % (opt,))
+        if key not in _MECHANISM_PROPS:
+            raise ValueError("%s is not a supported auth "
+                             "mechanism property. Must be one of "
+                             "%s." % (key, tuple(_MECHANISM_PROPS)))
+        props[key] = val
+
     return props
 
 
 def validate_document_class(option, value):
     """Validate the document_class option."""
-    if not issubclass(value, collections.MutableMapping):
-        raise TypeError("%s must be dict, bson.son.SON, or another "
+    if not issubclass(value, (collections.MutableMapping, RawBSONDocument)):
+        raise TypeError("%s must be dict, bson.son.SON, "
+                        "bson.raw_bson.RawBSONDocument, or a "
                         "sublass of collections.MutableMapping" % (option,))
     return value
 
@@ -366,11 +375,12 @@ def validate_is_mapping(option, value):
                         "collections.Mapping" % (option,))
 
 
-def validate_is_mutable_mapping(option, value):
-    """Validate the type of method arguments that expect a mutable document."""
-    if not isinstance(value, collections.MutableMapping):
-        raise TypeError("%s must be an instance of dict, bson.son.SON, or "
-                        "other type that inherits from "
+def validate_is_document_type(option, value):
+    """Validate the type of method arguments that expect a MongoDB document."""
+    if not isinstance(value, (collections.MutableMapping, RawBSONDocument)):
+        raise TypeError("%s must be an instance of dict, bson.son.SON, "
+                        "bson.raw_bson.RawBSONDocument, or "
+                        "a type that inherits from "
                         "collections.MutableMapping" % (option,))
 
 
@@ -378,7 +388,7 @@ def validate_ok_for_replace(replacement):
     """Validate a replacement document."""
     validate_is_mapping("replacement", replacement)
     # Replacement can be {}
-    if replacement:
+    if replacement and not isinstance(replacement, RawBSONDocument):
         first = next(iter(replacement))
         if first.startswith('$'):
             raise ValueError('replacement can not include $ operators')
@@ -417,6 +427,7 @@ VALIDATORS = {
     'ssl_cert_reqs': validate_cert_reqs,
     'ssl_ca_certs': validate_readable,
     'ssl_match_hostname': validate_boolean_or_string,
+    'readconcernlevel': validate_string_or_none,
     'read_preference': validate_read_preference,
     'readpreference': validate_read_preference_mode,
     'readpreferencetags': validate_read_preference_tags,
@@ -428,6 +439,8 @@ VALIDATORS = {
     'document_class': validate_document_class,
     'tz_aware': validate_boolean_or_string,
     'uuidrepresentation': validate_uuid_representation,
+    'connect': validate_boolean,
+    'event_listeners': _validate_event_listeners
 }
 
 
@@ -453,6 +466,23 @@ def validate(option, value):
     return lower, value
 
 
+def get_validated_options(options):
+    """Validate each entry in options and raise a warning if it is not valid.
+    Returns a copy of options with invalid entries removed
+    """
+    validated_options = {}
+    for opt, value in iteritems(options):
+        lower = opt.lower()
+        try:
+            validator = VALIDATORS.get(lower, raise_config_error)
+            value = validator(opt, value)
+        except (ValueError, ConfigurationError) as exc:
+            warnings.warn(str(exc))
+        else:
+            validated_options[lower] = value
+    return validated_options
+
+
 WRITE_CONCERN_OPTIONS = frozenset([
     'w',
     'wtimeout',
@@ -470,7 +500,8 @@ class BaseObject(object):
     SHOULD NOT BE USED BY DEVELOPERS EXTERNAL TO MONGODB.
     """
 
-    def __init__(self, codec_options, read_preference, write_concern):
+    def __init__(self, codec_options, read_preference, write_concern,
+                 read_concern):
 
         if not isinstance(codec_options, CodecOptions):
             raise TypeError("codec_options must be an instance of "
@@ -488,6 +519,11 @@ class BaseObject(object):
                             "pymongo.write_concern.WriteConcern")
         self.__write_concern = write_concern
 
+        if not isinstance(read_concern, ReadConcern):
+            raise TypeError("read_concern must be an instance of "
+                            "pymongo.read_concern.ReadConcern")
+        self.__read_concern = read_concern
+
     @property
     def codec_options(self):
         """Read only access to the :class:`~bson.codec_options.CodecOptions`
@@ -514,3 +550,10 @@ class BaseObject(object):
         """
         return self.__read_preference
 
+    @property
+    def read_concern(self):
+        """Read only access to the read concern of this instance.
+
+        .. versionadded:: 3.2
+        """
+        return self.__read_concern
