@@ -154,6 +154,7 @@ class Cursor(object):
         self.__hint = None
         self.__comment = None
         self.__max_time_ms = None
+        self.__max_await_time_ms = None
         self.__max = None
         self.__min = None
         self.__manipulate = manipulate
@@ -184,6 +185,7 @@ class Cursor(object):
 
         self.__codec_options = collection.codec_options
         self.__read_preference = collection.read_preference
+        self.__read_concern = collection.read_concern
 
         self.__query_flags = cursor_type
         if self.__read_preference != ReadPreference.PRIMARY:
@@ -243,10 +245,10 @@ class Cursor(object):
         """Internal clone helper."""
         clone = self._clone_base()
         values_to_clone = ("spec", "projection", "skip", "limit",
-                           "max_time_ms", "comment", "max", "min",
-                           "ordering", "explain", "hint", "batch_size",
-                           "max_scan", "manipulate", "query_flags",
-                           "modifiers")
+                           "max_time_ms", "max_await_time_ms", "comment",
+                           "max", "min", "ordering", "explain", "hint",
+                           "batch_size", "max_scan", "manipulate",
+                           "query_flags", "modifiers")
         data = dict((k, v) for k, v in iteritems(self.__dict__)
                     if k.startswith('_Cursor__') and k[9:] in values_to_clone)
         if deepcopy:
@@ -469,6 +471,34 @@ class Cursor(object):
         self.__max_time_ms = max_time_ms
         return self
 
+    def max_await_time_ms(self, max_await_time_ms):
+        """Specifies a time limit for a getMore operation on a
+        :attr:`~pymongo.cursor.CursorType.TAILABLE_AWAIT` cursor. For all other
+        types of cursor max_await_time_ms is ignored.
+
+        Raises :exc:`TypeError` if `max_await_time_ms` is not an integer or
+        ``None``. Raises :exc:`~pymongo.errors.InvalidOperation` if this
+        :class:`Cursor` has already been used.
+
+        .. note:: `max_await_time_ms` requires server version **>= 3.2**
+
+        :Parameters:
+          - `max_await_time_ms`: the time limit after which the operation is
+            aborted
+
+        .. versionadded:: 3.2
+        """
+        if (not isinstance(max_await_time_ms, integer_types)
+                and max_await_time_ms is not None):
+            raise TypeError("max_await_time_ms must be an integer or None")
+        self.__check_okay_to_chain()
+
+        # Ignore max_await_time_ms if not tailable or await_data is False.
+        if self.__query_flags & CursorType.TAILABLE_AWAIT:
+            self.__max_await_time_ms = max_await_time_ms
+
+        return self
+
     def __getitem__(self, index):
         """Get a single document or a slice of documents from this cursor.
 
@@ -805,6 +835,7 @@ class Cursor(object):
         client = self.__collection.database.client
         listeners = client._event_listeners
         publish = listeners.enabled_for_commands
+        from_command = False
 
         if operation:
             kwargs = {
@@ -827,6 +858,7 @@ class Cursor(object):
                 data = response.data
                 cmd_duration = response.duration
                 rqst_id = response.request_id
+                from_command = response.from_command
             except AutoReconnect:
                 # Don't try to send kill cursors on another socket
                 # or to another server. It can cause a _pinValue
@@ -869,6 +901,8 @@ class Cursor(object):
             doc = helpers._unpack_response(response=data,
                                            cursor_id=self.__id,
                                            codec_options=self.__codec_options)
+            if from_command:
+                helpers._check_command_response(doc['data'][0])
         except OperationFailure as exc:
             self.__killed = True
 
@@ -913,7 +947,9 @@ class Cursor(object):
         if publish:
             duration = (datetime.datetime.now() - start) + cmd_duration
             # Must publish in find / getMore / explain command response format.
-            if cmd_name == "explain":
+            if from_command:
+                res = doc['data'][0]
+            elif cmd_name == "explain":
                 res = doc["data"][0] if doc["number_returned"] else {}
             else:
                 res = {"cursor": {"id": doc["cursor_id"],
@@ -926,12 +962,23 @@ class Cursor(object):
             listeners.publish_command_success(
                 duration, res, cmd_name, rqst_id, self.__address)
 
-        self.__id = doc["cursor_id"]
+        if from_command and cmd_name != "explain":
+            cursor = doc['data'][0]['cursor']
+            self.__id = cursor['id']
+            if cmd_name == 'find':
+                documents = cursor['firstBatch']
+            else:
+                documents = cursor['nextBatch']
+            self.__data = deque(documents)
+            self.__retrieved += len(documents)
+        else:
+            self.__id = doc["cursor_id"]
+            self.__data = deque(doc["data"])
+            self.__retrieved += doc["number_returned"]
+
         if self.__id == 0:
             self.__killed = True
 
-        self.__retrieved += doc["number_returned"]
-        self.__data = deque(doc["data"])
 
         if self.__limit and self.__id and self.__limit <= self.__retrieved:
             self.__die()
@@ -959,7 +1006,8 @@ class Cursor(object):
                 else:
                     ntoreturn = self.__limit
             self.__send_message(_Query(self.__query_flags,
-                                       self.__collection.full_name,
+                                       self.__collection.database.name,
+                                       self.__collection.name,
                                        self.__skip,
                                        ntoreturn,
                                        self.__query_spec(),
@@ -967,7 +1015,8 @@ class Cursor(object):
                                        self.__codec_options,
                                        self.__read_preference,
                                        self.__limit,
-                                       self.__batch_size))
+                                       self.__batch_size,
+                                       self.__read_concern))
             if not self.__id:
                 self.__killed = True
         elif self.__id:  # Get More
@@ -982,10 +1031,12 @@ class Cursor(object):
             if self.__exhaust:
                 self.__send_message(None)
             else:
-                self.__send_message(_GetMore(self.__collection.full_name,
+                self.__send_message(_GetMore(self.__collection.database.name,
+                                             self.__collection.name,
                                              limit,
                                              self.__id,
-                                             self.__max_time_ms))
+                                             self.__codec_options,
+                                             self.__max_await_time_ms))
 
         else:  # Cursor id is zero nothing else to return
             self.__killed = True
