@@ -17,12 +17,12 @@
 import warnings
 
 from bson.code import Code
-from bson.codec_options import CodecOptions
+from bson.codec_options import CodecOptions, DEFAULT_CODEC_OPTIONS
 from bson.dbref import DBRef
 from bson.objectid import ObjectId
 from bson.py3compat import iteritems, string_type, _unicode
 from bson.son import SON
-from pymongo import auth, common
+from pymongo import auth, common, helpers
 from pymongo.collection import Collection
 from pymongo.command_cursor import CommandCursor
 from pymongo.errors import (CollectionInvalid,
@@ -51,8 +51,8 @@ class Database(common.BaseObject):
     """A Mongo database.
     """
 
-    def __init__(self, client, name, codec_options=None,
-                 read_preference=None, write_concern=None):
+    def __init__(self, client, name, codec_options=None, read_preference=None,
+                 write_concern=None, read_concern=None):
         """Get a database by client and name.
 
         Raises :class:`TypeError` if `name` is not an instance of
@@ -71,9 +71,15 @@ class Database(common.BaseObject):
           - `write_concern` (optional): An instance of
             :class:`~pymongo.write_concern.WriteConcern`. If ``None`` (the
             default) client.write_concern is used.
+          - `read_concern` (optional): An instance of
+            :class:`~pymongo.read_concern.ReadConcern`. If ``None`` (the
+            default) client.read_concern is used.
 
         .. mongodoc:: databases
 
+        .. versionchanged:: 3.2
+           Added the read_concern option.
+
         .. versionchanged:: 3.0
            Added the codec_options, read_preference, and write_concern options.
            :class:`~pymongo.database.Database` no longer returns an instance
@@ -89,7 +95,8 @@ class Database(common.BaseObject):
         super(Database, self).__init__(
             codec_options or client.codec_options,
             read_preference or client.read_preference,
-            write_concern or client.write_concern)
+            write_concern or client.write_concern,
+            read_concern or client.read_concern)
 
         if not isinstance(name, string_type):
             raise TypeError("name must be an instance "
@@ -225,8 +232,8 @@ class Database(common.BaseObject):
         """
         return Collection(self, name)
 
-    def get_collection(self, name, codec_options=None,
-                       read_preference=None, write_concern=None):
+    def get_collection(self, name, codec_options=None, read_preference=None,
+                       write_concern=None, read_concern=None):
         """Get a :class:`~pymongo.collection.Collection` with the given name
         and options.
 
@@ -259,12 +266,27 @@ class Database(common.BaseObject):
             :class:`~pymongo.write_concern.WriteConcern`. If ``None`` (the
             default) the :attr:`write_concern` of this :class:`Database` is
             used.
+          - `read_concern` (optional): An instance of
+            :class:`~pymongo.read_concern.ReadConcern`. If ``None`` (the
+            default) the :attr:`read_concern` of this :class:`Database` is
+            used.
         """
         return Collection(
-            self, name, False, codec_options, read_preference, write_concern)
+            self, name, False, codec_options, read_preference,
+            write_concern, read_concern)
+
+    def _collection_default_options(self, name, **kargs):
+        """Get a Collection instance with the default settings."""
+        wc = (self.write_concern
+              if self.write_concern.acknowledged else WriteConcern())
+        return self.get_collection(
+            name, codec_options=DEFAULT_CODEC_OPTIONS,
+            read_preference=ReadPreference.PRIMARY,
+            write_concern=wc)
 
     def create_collection(self, name, codec_options=None,
-                          read_preference=None, write_concern=None, **kwargs):
+                          read_preference=None, write_concern=None,
+                          read_concern=None, **kwargs):
         """Create a new :class:`~pymongo.collection.Collection` in this
         database.
 
@@ -298,6 +320,10 @@ class Database(common.BaseObject):
             :class:`~pymongo.write_concern.WriteConcern`. If ``None`` (the
             default) the :attr:`write_concern` of this :class:`Database` is
             used.
+          - `read_concern` (optional): An instance of
+            :class:`~pymongo.read_concern.ReadConcern`. If ``None`` (the
+            default) the :attr:`read_concern` of this :class:`Database` is
+            used.
           - `**kwargs` (optional): additional keyword arguments will
             be passed as options for the create collection command
 
@@ -311,7 +337,8 @@ class Database(common.BaseObject):
             raise CollectionInvalid("collection %s already exists" % name)
 
         return Collection(self, name, True, codec_options,
-                          read_preference, write_concern, **kwargs)
+                          read_preference, write_concern,
+                          read_concern, **kwargs)
 
     def _apply_incoming_manipulators(self, son, collection):
         """Apply incoming manipulators to `son`."""
@@ -351,7 +378,7 @@ class Database(common.BaseObject):
 
     def _command(self, sock_info, command, slave_ok=False, value=1, check=True,
                  allowable_errors=None, read_preference=ReadPreference.PRIMARY,
-                 codec_options=CodecOptions(), **kwargs):
+                 codec_options=DEFAULT_CODEC_OPTIONS, **kwargs):
         """Internal command helper."""
         if isinstance(command, string_type):
             command = SON([(command, value)])
@@ -367,7 +394,7 @@ class Database(common.BaseObject):
 
     def command(self, command, value=1, check=True,
                 allowable_errors=None, read_preference=ReadPreference.PRIMARY,
-                codec_options=CodecOptions(), **kwargs):
+                codec_options=DEFAULT_CODEC_OPTIONS, **kwargs):
         """Issue a MongoDB command.
 
         Send command `command` to the database and return the
@@ -466,7 +493,7 @@ class Database(common.BaseObject):
             return CommandCursor(coll, cursor, sock_info.address)
         else:
             coll = self["system.namespaces"]
-            res = _first_batch(sock_info, coll.full_name,
+            res = _first_batch(sock_info, coll.database.name, coll.name,
                                criteria, 0, slave_okay,
                                CodecOptions(), ReadPreference.PRIMARY, cmd,
                                self.client._event_listeners)
@@ -592,12 +619,16 @@ class Database(common.BaseObject):
           - `include_all` (optional): if ``True`` also list currently
             idle operations in the result
         """
-        coll = self.get_collection(
-            "$cmd.sys.inprog", read_preference=ReadPreference.PRIMARY)
-        if include_all:
-            return coll.find_one({"$all": True})
-        else:
-            return coll.find_one()
+        cmd = SON([("currentOp", 1), ("$all", include_all)])
+        with self.__client._socket_for_writes() as sock_info:
+            if sock_info.max_wire_version >= 4:
+                return sock_info.command("admin", cmd)
+            else:
+                spec = {"$all": True} if include_all else {}
+                x = helpers._first_batch(sock_info, "admin", "$cmd.sys.inprog",
+                    spec, -1, True, self.codec_options,
+                    ReadPreference.PRIMARY, cmd, self.client._event_listeners)
+                return x.get('data', [None])[0]
 
     def profiling_level(self):
         """Get the database's current profiling level.
@@ -802,7 +833,9 @@ class Database(common.BaseObject):
     def _legacy_add_user(self, name, password, read_only, **kwargs):
         """Uses v1 system to add users, i.e. saving to system.users.
         """
-        user = self.system.users.find_one({"user": name}) or {"user": name}
+        # Use a Collection with the default codec_options.
+        system_users = self._collection_default_options('system.users')
+        user = system_users.find_one({"user": name}) or {"user": name}
         if password is not None:
             user["pwd"] = auth._password_digest(name, password)
         if read_only is not None:
@@ -812,11 +845,8 @@ class Database(common.BaseObject):
         # We don't care what the _id is, only that it has one
         # for the replace_one call below.
         user.setdefault("_id", ObjectId())
-        coll = self.system.users
-        if not self.write_concern.acknowledged:
-            coll = coll.with_options(write_concern=WriteConcern())
         try:
-            coll.replace_one({"_id": user["_id"]}, user, True)
+            system_users.replace_one({"_id": user["_id"]}, user, True)
         except OperationFailure as exc:
             # First admin user add fails gle in MongoDB >= 2.1.2
             # See SERVER-4225 for more information.
@@ -908,9 +938,7 @@ class Database(common.BaseObject):
         except OperationFailure as exc:
             # See comment in add_user try / except above.
             if exc.code in common.COMMAND_NOT_FOUND_CODES:
-                coll = self.system.users
-                if not self.write_concern.acknowledged:
-                    coll = coll.with_options(write_concern=WriteConcern())
+                coll = self._collection_default_options('system.users')
                 coll.delete_one({"user": name})
                 return
             raise
