@@ -15,14 +15,14 @@
 """Bits and pieces used by the driver that don't really fit elsewhere."""
 
 import collections
+import datetime
 import struct
-from pymongo.message import _Query
 
 import bson
-import pymongo
 from bson.codec_options import CodecOptions
 from bson.py3compat import itervalues, string_type, iteritems, u
 from bson.son import SON
+from pymongo import ASCENDING
 from pymongo.errors import (CursorNotFound,
                             DuplicateKeyError,
                             ExecutionTimeout,
@@ -31,6 +31,8 @@ from pymongo.errors import (CursorNotFound,
                             WriteError,
                             WriteConcernError,
                             WTimeoutError)
+from pymongo.message import _Query, _convert_exception
+from pymongo.read_concern import DEFAULT_READ_CONCERN
 
 
 _UUNDER = u("_")
@@ -50,7 +52,7 @@ def _index_list(key_or_list, direction=None):
         return [(key_or_list, direction)]
     else:
         if isinstance(key_or_list, string_type):
-            return [(key_or_list, pymongo.ASCENDING)]
+            return [(key_or_list, ASCENDING)]
         elif not isinstance(key_or_list, (list, tuple)):
             raise TypeError("if no direction is specified, "
                             "key_or_list must be an instance of list")
@@ -106,12 +108,16 @@ def _unpack_response(response, cursor_id=None, codec_options=CodecOptions()):
         # Shouldn't get this response if we aren't doing a getMore
         assert cursor_id is not None
 
-        raise CursorNotFound("cursor id '%s' not valid at server" %
-                             cursor_id)
+        # Fake a getMore command response. OP_GET_MORE provides no document.
+        msg = "Cursor not found, cursor id: %d" % (cursor_id,)
+        errobj = {"ok": 0, "errmsg": msg, "code": 43}
+        raise CursorNotFound(msg, 43, errobj)
     elif response_flag & 2:
         error_object = bson.BSON(response[20:]).decode()
+        # Fake the ok field if it doesn't exist.
+        error_object.setdefault("ok", 0)
         if error_object["$err"].startswith("not master"):
-            raise NotMasterError(error_object["$err"])
+            raise NotMasterError(error_object["$err"], error_object)
         elif error_object.get("code") == 50:
             raise ExecutionTimeout(error_object.get("$err"),
                                    error_object.get("code"),
@@ -121,11 +127,11 @@ def _unpack_response(response, cursor_id=None, codec_options=CodecOptions()):
                                error_object.get("code"),
                                error_object)
 
-    result = {}
-    result["cursor_id"] = struct.unpack("<q", response[4:12])[0]
-    result["starting_from"] = struct.unpack("<i", response[12:16])[0]
-    result["number_returned"] = struct.unpack("<i", response[16:20])[0]
-    result["data"] = bson.decode_all(response[20:], codec_options)
+    result = {"cursor_id": struct.unpack("<q", response[4:12])[0],
+              "starting_from": struct.unpack("<i", response[12:16])[0],
+              "number_returned": struct.unpack("<i", response[16:20])[0],
+              "data": bson.decode_all(response[20:], codec_options)}
+
     assert len(result["data"]) == result["number_returned"]
     return result
 
@@ -166,7 +172,7 @@ def _check_command_response(response, msg=None, allowable_errors=None):
             # Server is "not master" or "recovering"
             if (errmsg.startswith("not master")
                     or errmsg.startswith("node is recovering")):
-                raise NotMasterError(errmsg)
+                raise NotMasterError(errmsg, response)
 
             # Server assertion failures
             if errmsg == "db assertion failure":
@@ -183,6 +189,8 @@ def _check_command_response(response, msg=None, allowable_errors=None):
                 raise DuplicateKeyError(errmsg, code, response)
             elif code == 50:
                 raise ExecutionTimeout(errmsg, code, response)
+            elif code == 43:
+                raise CursorNotFound(errmsg, code, response)
 
             msg = msg or "%s"
             raise OperationFailure(msg % errmsg, code, response)
@@ -211,7 +219,7 @@ def _check_gle_response(response):
         return result
 
     if error_msg.startswith("not master"):
-        raise NotMasterError(error_msg)
+        raise NotMasterError(error_msg, result)
 
     details = result
 
@@ -228,16 +236,48 @@ def _check_gle_response(response):
     raise OperationFailure(details["err"], code, result)
 
 
-def _first_batch(sock_info, namespace, query,
-                 limit, slave_ok, codec_options, read_preference):
+def _first_batch(sock_info, db, coll, query, ntoreturn,
+                 slave_ok, codec_options, read_preference, cmd, listeners):
     """Simple query helper for retrieving a first (and possibly only) batch."""
     query = _Query(
-        0, namespace, 0, limit, query, None, codec_options, read_preference)
+        0, db, coll, 0, ntoreturn, query, None,
+        codec_options, read_preference, 0, 0, DEFAULT_READ_CONCERN)
+
+    name = next(iter(cmd))
+    duration = None
+    publish = listeners.enabled_for_commands
+    if publish:
+        start = datetime.datetime.now()
+
     request_id, msg, max_doc_size = query.get_message(slave_ok,
                                                       sock_info.is_mongos)
+
+    if publish:
+        encoding_duration = datetime.datetime.now() - start
+        listeners.publish_command_start(
+            cmd, db, request_id, sock_info.address)
+        start = datetime.datetime.now()
+
     sock_info.send_message(msg, max_doc_size)
     response = sock_info.receive_message(1, request_id)
-    return _unpack_response(response, None, codec_options)
+    try:
+        result = _unpack_response(response, None, codec_options)
+    except Exception as exc:
+        if publish:
+            duration = (datetime.datetime.now() - start) + encoding_duration
+            if isinstance(exc, (NotMasterError, OperationFailure)):
+                failure = exc.details
+            else:
+                failure = _convert_exception(exc)
+            listeners.publish_command_failure(
+                duration, failure, name, request_id, sock_info.address)
+        raise
+    if publish:
+        duration = (datetime.datetime.now() - start) + encoding_duration
+        listeners.publish_command_success(
+            duration, result, name, request_id, sock_info.address)
+
+    return result
 
 
 def _check_write_command_response(results):
