@@ -38,10 +38,13 @@ import warnings
 import weakref
 from collections import defaultdict
 
+from bson.codec_options import DEFAULT_CODEC_OPTIONS
 from bson.py3compat import (integer_types,
                             string_type)
+from bson.son import SON
 from pymongo import (common,
                      database,
+                     helpers,
                      message,
                      periodic_executor,
                      uri_parser)
@@ -62,11 +65,15 @@ from pymongo.server_type import SERVER_TYPE
 from pymongo.topology import Topology
 from pymongo.topology_description import TOPOLOGY_TYPE
 from pymongo.settings import TopologySettings
+from pymongo.write_concern import WriteConcern
 
 
 class MongoClient(common.BaseObject):
     HOST = "localhost"
     PORT = 27017
+    # Define order to retrieve options from ClientOptions for __repr__.
+    # No host/port; these are retrieved from TopologySettings.
+    _constructor_args = ('document_class', 'tz_aware', 'connect')
 
     def __init__(
             self,
@@ -95,6 +102,9 @@ class MongoClient(common.BaseObject):
         passwords reserved characters like ':', '/', '+' and '@' must be
         escaped following RFC 2396.
 
+        .. warning:: When using PyMongo in a multiprocessing context, please
+          read :ref:`multiprocessing` first.
+
         :Parameters:
           - `host` (optional): hostname or IP address of the
             instance to connect to, or a mongodb URI, or a list of
@@ -141,6 +151,8 @@ class MongoClient(common.BaseObject):
           - `socketKeepAlive`: (boolean) Whether to send periodic keep-alive
             packets on connected sockets. Defaults to ``False`` (do not send
             keep-alive packets).
+          - `event_listeners`: a list or tuple of event listeners. See
+            :mod:`~pymongo.monitoring` for details.
 
           | **Write Concern options:**
           | (Only set if passed. No default values.)
@@ -175,10 +187,7 @@ class MongoClient(common.BaseObject):
             match this name. Implies that the hosts specified are a seed list
             and the driver should attempt to find all members of the set.
             Defaults to ``None``.
-          - `read_preference`: The read preference for this client. If
-            connecting directly to a secondary then a read preference mode
-            *other* than PRIMARY is required - otherwise all queries will throw
-            :class:`~pymongo.errors.AutoReconnect` "not master".
+          - `read_preference`: The read preference for this client.
             See :class:`~pymongo.read_preferences.ReadPreference` for all
             available read preference options. Defaults to ``PRIMARY``.
 
@@ -216,6 +225,15 @@ class MongoClient(common.BaseObject):
             this to ``False`` as that could make your application vulnerable to
             man-in-the-middle attacks.
 
+          | **Read Concern options:**
+          | (If not set explicitly, this will use the server default)
+
+          - `readConcernLevel`: (string) The read concern level specifies the
+            level of isolation for read operations.  For example, a read
+            operation using a read concern level of ``majority`` will only
+            return data that has been written to a majority of nodes. If the
+            level is left unspecified, the server default will be used.
+
         .. mongodoc:: connections
 
         .. versionchanged:: 3.0
@@ -294,7 +312,7 @@ class MongoClient(common.BaseObject):
         for entity in host:
             if "://" in entity:
                 if entity.startswith("mongodb://"):
-                    res = uri_parser.parse_uri(entity, port, False)
+                    res = uri_parser.parse_uri(entity, port, warn=True)
                     seeds.update(res["nodelist"])
                     username = res["username"] or username
                     password = res["password"] or password
@@ -315,9 +333,14 @@ class MongoClient(common.BaseObject):
         monitor_class = kwargs.pop('_monitor_class', None)
         condition_class = kwargs.pop('_condition_class', None)
 
-        kwargs['document_class'] = document_class
-        kwargs['tz_aware'] = tz_aware
-        opts.update(kwargs)
+        keyword_opts = kwargs
+        keyword_opts['document_class'] = document_class
+        keyword_opts['tz_aware'] = tz_aware
+        keyword_opts['connect'] = connect
+        # Validate all keyword options.
+        keyword_opts = dict(common.validate(k, v)
+                            for k, v in keyword_opts.items())
+        opts.update(keyword_opts)
         self.__options = options = ClientOptions(
             username, password, dbase, opts)
 
@@ -326,12 +349,15 @@ class MongoClient(common.BaseObject):
         self.__cursor_manager = CursorManager(self)
         self.__kill_cursors_queue = []
 
+        self._event_listeners = options.pool_options.event_listeners
+
         # Cache of existing indexes used by ensure_index ops.
         self.__index_cache = {}
 
         super(MongoClient, self).__init__(options.codec_options,
                                           options.read_preference,
-                                          options.write_concern)
+                                          options.write_concern,
+                                          options.read_concern)
 
         self.__all_credentials = {}
         creds = options.credentials
@@ -360,10 +386,10 @@ class MongoClient(common.BaseObject):
             return True
 
         executor = periodic_executor.PeriodicExecutor(
-            condition_class=self._topology_settings.condition_class,
             interval=common.KILL_CURSOR_FREQUENCY,
-            min_interval=0,
-            target=target)
+            min_interval=0.5,
+            target=target,
+            name="pymongo_kill_cursors_thread")
 
         # We strongly reference the executor and it weakly references us via
         # this closure. When the client is freed, stop the executor soon.
@@ -454,22 +480,29 @@ class MongoClient(common.BaseObject):
         if index_name in self.__index_cache[database_name][collection_name]:
             del self.__index_cache[database_name][collection_name][index_name]
 
-    def _server_property(self, attr_name, default=None):
+    def _server_property(self, attr_name):
         """An attribute of the current server's description.
 
-        Returns "default" while there is no current server, primary, or mongos.
+        If the client is not connected, this will block until a connection is
+        established or raise ServerSelectionTimeoutError if no server is
+        available.
 
         Not threadsafe if used multiple times in a single method, since
         the server may change. In such cases, store a local reference to a
         ServerDescription first, then use its properties.
         """
-        try:
-            server = self._topology.select_server(
-                writable_server_selector, server_selection_timeout=0)
+        server = self._topology.select_server(
+            writable_server_selector)
 
-            return getattr(server.description, attr_name)
-        except ConnectionFailure:
-            return default
+        return getattr(server.description, attr_name)
+
+    @property
+    def event_listeners(self):
+        """The event listeners registered for this client.
+
+        See :mod:`~pymongo.monitoring` for details.
+        """
+        return self._event_listeners.event_listeners
 
     @property
     def address(self):
@@ -479,15 +512,21 @@ class MongoClient(common.BaseObject):
         the client is load-balancing among mongoses, since there is no single
         address. Use :attr:`nodes` instead.
 
+        If the client is not connected, this will block until a connection is
+        established or raise ServerSelectionTimeoutError if no server is
+        available.
+
         .. versionadded:: 3.0
         """
-        try:
-            return self._topology.get_direct_or_primary()
-        except InvalidOperation:
-            # Only one case where Topology throws InvalidOperation.
+        topology_type = self._topology._description.topology_type
+        if topology_type == TOPOLOGY_TYPE.Sharded:
             raise InvalidOperation(
                 'Cannot use "address" property when load balancing among'
                 ' mongoses, use "nodes" instead.')
+        if topology_type not in (TOPOLOGY_TYPE.ReplicaSetWithPrimary,
+                                 TOPOLOGY_TYPE.Single):
+            return None
+        return self._server_property('address')
 
     @property
     def primary(self):
@@ -529,16 +568,20 @@ class MongoClient(common.BaseObject):
 
     @property
     def is_primary(self):
-        """If this client if connected to a server that can accept writes.
+        """If this client is connected to a server that can accept writes.
 
         True if the current server is a standalone, mongos, or the primary of
-        a replica set.
+        a replica set. If the client is not connected, this will block until a
+        connection is established or raise ServerSelectionTimeoutError if no
+        server is available.
         """
-        return self._server_property('is_writable', False)
+        return self._server_property('is_writable')
 
     @property
     def is_mongos(self):
-        """If this client is connected to mongos.
+        """If this client is connected to mongos. If the client is not
+        connected, this will block until a connection is established or raise
+        ServerSelectionTimeoutError if no server is available..
         """
         return self._server_property('server_type') == SERVER_TYPE.Mongos
 
@@ -571,28 +614,34 @@ class MongoClient(common.BaseObject):
     def max_bson_size(self):
         """The largest BSON object the connected server accepts in bytes.
 
-        Defaults to 16MB if not connected to a server.
+        If the client is not connected, this will block until a connection is
+        established or raise ServerSelectionTimeoutError if no server is
+        available.
         """
-        return self._server_property('max_bson_size', common.MAX_BSON_SIZE)
+        return self._server_property('max_bson_size')
 
     @property
     def max_message_size(self):
         """The largest message the connected server accepts in bytes.
 
-        Defaults to 32MB if not connected to a server.
+        If the client is not connected, this will block until a connection is
+        established or raise ServerSelectionTimeoutError if no server is
+        available.
         """
-        return self._server_property(
-            'max_message_size', common.MAX_MESSAGE_SIZE)
+        return self._server_property('max_message_size')
 
     @property
     def max_write_batch_size(self):
         """The maxWriteBatchSize reported by the server.
 
+        If the client is not connected, this will block until a connection is
+        established or raise ServerSelectionTimeoutError if no server is
+        available.
+
         Returns a default value when connected to server versions prior to
         MongoDB 2.6.
         """
-        return self._server_property(
-            'max_write_batch_size', common.MAX_WRITE_BATCH_SIZE)
+        return self._server_property('max_write_batch_size')
 
     @property
     def local_threshold_ms(self):
@@ -741,6 +790,7 @@ class MongoClient(common.BaseObject):
             operation,
             set_slave_ok,
             self.__all_credentials,
+            self._event_listeners,
             exhaust)
 
     def _reset_on_error(self, server, func, *args, **kwargs):
@@ -776,14 +826,37 @@ class MongoClient(common.BaseObject):
     def __ne__(self, other):
         return not self == other
 
+    def _repr_helper(self):
+        def option_repr(option, value):
+            """Fix options whose __repr__ isn't usable in a constructor."""
+            if option == 'document_class':
+                if value is dict:
+                    return 'document_class=dict'
+                else:
+                    return 'document_class=%s.%s' % (value.__module__,
+                                                     value.__name__)
+            if "ms" in option:
+                return "%s='%s'" % (option, int(value * 1000))
+
+            return '%s=%r' % (option, value)
+
+        # Host first...
+        options = ['host=%r' % [
+            '%s:%d' % (host, port)
+            for host, port in self._topology_settings.seeds]]
+        # ... then everything in self._constructor_args...
+        options.extend(
+            option_repr(key, self.__options._options[key])
+            for key in self._constructor_args)
+        # ... then everything else.
+        options.extend(
+            option_repr(key, self.__options._options[key])
+            for key in self.__options._options
+            if key not in set(self._constructor_args))
+        return ', '.join(options)
+
     def __repr__(self):
-        server_descriptions = self._topology.description.server_descriptions()
-        if len(server_descriptions) == 1:
-            description, = server_descriptions.values()
-            return "MongoClient(%r, %r)" % description.address
-        else:
-            return "MongoClient(%r)" % [
-                "%s:%d" % address for address in server_descriptions]
+        return ("MongoClient(%s)" % (self._repr_helper(),))
 
     def __getattr__(self, name):
         """Get a database by name.
@@ -876,18 +949,64 @@ class MongoClient(common.BaseObject):
 
         # Don't re-open topology if it's closed and there's no pending cursors.
         if address_to_cursor_ids:
+            listeners = self._event_listeners
+            publish = listeners.enabled_for_commands
             topology = self._get_topology()
             for address, cursor_ids in address_to_cursor_ids.items():
                 try:
                     if address:
-                        server = topology.select_server_by_address(address)
+                        # address could be a tuple or _CursorAddress, but
+                        # select_server_by_address needs (host, port).
+                        server = topology.select_server_by_address(
+                            tuple(address))
                     else:
                         # Application called close_cursor() with no address.
                         server = topology.select_server(
                             writable_server_selector)
 
-                    server.send_message(message.kill_cursors(cursor_ids),
-                                        self.__all_credentials)
+                    try:
+                        namespace = address.namespace
+                        db, coll = namespace.split('.', 1)
+                    except AttributeError:
+                        namespace = None
+                        db = coll = "OP_KILL_CURSORS"
+
+                    spec = SON([('killCursors', coll),
+                                ('cursors', cursor_ids)])
+                    with server.get_socket(self.__all_credentials) as sock_info:
+                        if (sock_info.max_wire_version >= 4 and
+                                namespace is not None):
+                            sock_info.command(db, spec)
+                        else:
+                            if publish:
+                                start = datetime.datetime.now()
+                            request_id, msg = message.kill_cursors(cursor_ids)
+                            if publish:
+                                duration = datetime.datetime.now() - start
+                                listeners.publish_command_start(
+                                    spec, db, request_id, address)
+                                start = datetime.datetime.now()
+
+                            try:
+                                sock_info.send_message(msg, 0)
+                            except Exception as exc:
+                                if publish:
+                                    dur = ((datetime.datetime.now() - start)
+                                           + duration)
+                                    listeners.publish_command_failure(
+                                        dur, message._convert_exception(exc),
+                                        'killCursors', request_id, address)
+                                raise
+
+                            if publish:
+                                duration = ((datetime.datetime.now() - start)
+                                            + duration)
+                                # OP_KILL_CURSORS returns no reply, fake one.
+                                reply = {'cursorsUnknown': cursor_ids, 'ok': 1}
+                                listeners.publish_command_success(
+                                    duration, reply, 'killCursors', request_id,
+                                    address)
+
                 except ConnectionFailure as exc:
                     warnings.warn("couldn't close cursor on %s: %s"
                                   % (address, exc))
@@ -900,9 +1019,8 @@ class MongoClient(common.BaseObject):
     def database_names(self):
         """Get a list of the names of all databases on the connected server."""
         return [db["name"] for db in
-                self.admin.command(
-                    "listDatabases",
-                    read_preference=ReadPreference.PRIMARY)["databases"]]
+                self._database_default_options('admin').command(
+                    "listDatabases")["databases"]]
 
     def drop_database(self, name_or_database):
         """Drop a database.
@@ -944,8 +1062,8 @@ class MongoClient(common.BaseObject):
 
         return self[self.__default_database_name]
 
-    def get_database(self, name, codec_options=None,
-                     read_preference=None, write_concern=None):
+    def get_database(self, name, codec_options=None, read_preference=None,
+                     write_concern=None, read_concern=None):
         """Get a :class:`~pymongo.database.Database` with the given name and
         options.
 
@@ -978,9 +1096,21 @@ class MongoClient(common.BaseObject):
             :class:`~pymongo.write_concern.WriteConcern`. If ``None`` (the
             default) the :attr:`write_concern` of this :class:`MongoClient` is
             used.
+          - `read_concern` (optional): An instance of
+            :class:`~pymongo.read_concern.ReadConcern`. If ``None`` (the
+            default) the :attr:`read_concern` of this :class:`MongoClient` is
+            used.
         """
         return database.Database(
-            self, name, codec_options, read_preference, write_concern)
+            self, name, codec_options, read_preference,
+            write_concern, read_concern)
+
+    def _database_default_options(self, name):
+        """Get a Database instance with the default settings."""
+        return self.get_database(
+            name, codec_options=DEFAULT_CODEC_OPTIONS,
+            read_preference=ReadPreference.PRIMARY,
+            write_concern=WriteConcern())
 
     @property
     def is_locked(self):
@@ -988,7 +1118,7 @@ class MongoClient(common.BaseObject):
         are blocked, although read operations may still be allowed.
         Use :meth:`unlock` to unlock.
         """
-        ops = self.admin.current_op()
+        ops = self._database_default_options('admin').current_op()
         return bool(ops.get('fsyncLock', 0))
 
     def fsync(self, **kwargs):
@@ -1013,9 +1143,19 @@ class MongoClient(common.BaseObject):
     def unlock(self):
         """Unlock a previously locked server.
         """
-        coll = self.admin.get_collection(
-            "$cmd.sys.unlock", read_preference=ReadPreference.PRIMARY)
-        coll.find_one()
+        cmd = {"fsyncUnlock": 1}
+        with self._socket_for_writes() as sock_info:
+            if sock_info.max_wire_version >= 4:
+                try:
+                    sock_info.command("admin", cmd)
+                except OperationFailure as exc:
+                    # Ignore "DB not locked" to replicate old behavior
+                    if exc.code != 125:
+                        raise
+            else:
+                helpers._first_batch(sock_info, "admin", "$cmd.sys.unlock",
+                    {}, -1, True, self.codec_options,
+                    ReadPreference.PRIMARY, cmd, self._event_listeners)
 
     def __enter__(self):
         return self
