@@ -14,8 +14,10 @@
 
 """Internal class to monitor a topology of one or more servers."""
 
+import os
 import random
 import threading
+import warnings
 
 from bson.py3compat import itervalues
 from pymongo import common
@@ -50,13 +52,31 @@ class Topology(object):
         self._lock = threading.Lock()
         self._condition = self._settings.condition_class(self._lock)
         self._servers = {}
+        self._pid = None
 
     def open(self):
         """Start monitoring, or restart after a fork.
 
         No effect if called multiple times.
+
+        .. warning:: To avoid a deadlock during Python's getaddrinfo call,
+          will generate a warning if open() is called from a different
+          process than the one that initialized the Topology. To prevent this
+          from happening, MongoClient must be created after any forking OR
+          MongoClient must be started with connect=False.
         """
         with self._lock:
+            if self._pid is None:
+                self._pid = os.getpid()
+            else:
+                if os.getpid() != self._pid:
+                    warnings.warn(
+                        "MongoClient opened before fork. Create MongoClient "
+                        "with connect=False, or create client after forking. "
+                        "See PyMongo's documentation for details: http://api."
+                        "mongodb.org/python/current/faq.html#using-pymongo-"
+                        "with-multiprocessing>")
+
             self._ensure_opened()
 
     def select_servers(self,
@@ -198,22 +218,6 @@ class Topology(object):
             descriptions = selector(self._description.known_servers)
             return set([d.address for d in descriptions])
 
-    def get_direct_or_primary(self):
-        """Return the address of a connected primary or standalone, or None.
-
-        Raise InvalidOperation for Sharded topologies.
-        """
-        # Implemented here in Topology instead of MongoClient, so it can lock.
-        with self._lock:
-            topology_type = self._description.topology_type
-            if topology_type == TOPOLOGY_TYPE.Sharded:
-                raise InvalidOperation()
-            if topology_type not in (TOPOLOGY_TYPE.ReplicaSetWithPrimary,
-                                     TOPOLOGY_TYPE.Single):
-                return None
-            descriptions = writable_server_selector(
-                self._description.known_servers)
-            return descriptions[0].address if descriptions else None
 
     def get_secondaries(self):
         """Return set of secondary addresses."""
