@@ -29,6 +29,7 @@ from bson import (BSON,
                   decode_all,
                   decode_file_iter,
                   decode_iter,
+                  EPOCH_AWARE,
                   is_valid,
                   Regex)
 from bson.binary import Binary, UUIDLegacy
@@ -40,6 +41,7 @@ from bson.dbref import DBRef
 from bson.py3compat import PY3, u, text_type, iteritems, StringIO
 from bson.son import SON
 from bson.timestamp import Timestamp
+from bson.tz_util import FixedOffset
 from bson.errors import (InvalidBSON,
                          InvalidDocument,
                          InvalidStringData)
@@ -87,6 +89,29 @@ class NotADict(collections.MutableMapping):
         return "NotADict(%s)" % repr(self._dict)
 
 
+class DSTAwareTimezone(datetime.tzinfo):
+
+    def __init__(self, offset, name, dst_start_month, dst_end_month):
+        self.__offset = offset
+        self.__dst_start_month = dst_start_month
+        self.__dst_end_month = dst_end_month
+        self.__name = name
+
+    def _is_dst(self, dt):
+        return self.__dst_start_month <= dt.month <= self.__dst_end_month
+
+    def utcoffset(self, dt):
+        return datetime.timedelta(minutes=self.__offset) + self.dst(dt)
+
+    def dst(self, dt):
+        if self._is_dst(dt):
+            return datetime.timedelta(hours=1)
+        return datetime.timedelta(0)
+
+    def tzname(self, dt):
+        return self.__name
+
+
 class TestBSON(unittest.TestCase):
     def assertInvalid(self, data):
         self.assertRaises(InvalidBSON, bson.BSON(data).decode)
@@ -437,6 +462,58 @@ class TestBSON(unittest.TestCase):
         self.assertEqual(utc, after.tzinfo)
         self.assertEqual(as_utc, after)
 
+    def test_local_datetime(self):
+        # Timezone -60 minutes of UTC, with DST between April and July.
+        tz = DSTAwareTimezone(60, "sixty-minutes", 4, 7)
+
+        # It's not DST.
+        local = datetime.datetime(year=2025, month=12, hour=2, day=1,
+                                  tzinfo=tz)
+        options = CodecOptions(tz_aware=True, tzinfo=tz)
+        # Encode with this timezone, then decode to UTC.
+        encoded = BSON.encode({'date': local}, codec_options=options)
+        self.assertEqual(local.replace(hour=1, tzinfo=None),
+                         encoded.decode()['date'])
+
+        # It's DST.
+        local = datetime.datetime(year=2025, month=4, hour=1, day=1,
+                                  tzinfo=tz)
+        encoded = BSON.encode({'date': local}, codec_options=options)
+        self.assertEqual(local.replace(month=3, day=31, hour=23, tzinfo=None),
+                         encoded.decode()['date'])
+
+        # Encode UTC, then decode in a different timezone.
+        encoded = BSON.encode({'date': local.replace(tzinfo=utc)})
+        decoded = encoded.decode(options)['date']
+        self.assertEqual(local.replace(hour=3), decoded)
+        self.assertEqual(tz, decoded.tzinfo)
+
+        # Test round-tripping.
+        self.assertEqual(
+            local,
+            (BSON
+             .encode({'date': local}, codec_options=options)
+             .decode(options)['date']))
+
+        # Test around the Unix Epoch.
+        epochs = (
+            EPOCH_AWARE,
+            EPOCH_AWARE.astimezone(FixedOffset(120, 'one twenty')),
+            EPOCH_AWARE.astimezone(FixedOffset(-120, 'minus one twenty'))
+        )
+        utc_co = CodecOptions(tz_aware=True)
+        for epoch in epochs:
+            doc = {'epoch': epoch}
+            # We always retrieve datetimes in UTC unless told to do otherwise.
+            self.assertEqual(
+                EPOCH_AWARE,
+                BSON.encode(doc).decode(codec_options=utc_co)['epoch'])
+            # Round-trip the epoch.
+            local_co = CodecOptions(tz_aware=True, tzinfo=epoch.tzinfo)
+            self.assertEqual(
+                epoch,
+                BSON.encode(doc).decode(codec_options=local_co)['epoch'])
+
     def test_naive_decode(self):
         aware = datetime.datetime(1993, 4, 4, 2,
                                   tzinfo=FixedOffset(555, "SomeZone"))
@@ -462,8 +539,17 @@ class TestBSON(unittest.TestCase):
         evil_list['a'].append(evil_list)
         evil_dict = {}
         evil_dict['a'] = evil_dict
-        for evil_data in [evil_dict, evil_list]:
-            self.assertRaises(RuntimeError, BSON.encode, evil_data)
+        # Work around what seems like a regression in python 3.5.0.
+        # See http://bugs.python.org/issue25222
+        # 100 is an arbitrary choice. The default is 1000 on the machines
+        # I have access to.
+        depth = sys.getrecursionlimit()
+        sys.setrecursionlimit(100)
+        try:
+            for evil_data in [evil_dict, evil_list]:
+                self.assertRaises(RuntimeError, BSON.encode, evil_data)
+        finally:
+            sys.setrecursionlimit(depth)
 
     def test_overflow(self):
         self.assertTrue(BSON.encode({"x": long(9223372036854775807)}))
@@ -654,14 +740,14 @@ class TestBSON(unittest.TestCase):
         self.assertEqual(doc1, BSON(doc1_bson).decode())
 
         # Valid Python regex, with flags.
-        re2 = re.compile('.*', re.I | re.L | re.M | re.S | re.U | re.X)
-        bson_re2 = Regex('.*', re.I | re.L | re.M | re.S | re.U | re.X)
+        re2 = re.compile(u('.*'), re.I | re.M | re.S | re.U | re.X)
+        bson_re2 = Regex(u('.*'), re.I | re.M | re.S | re.U | re.X)
 
         doc2_with_re = {'r': re2}
         doc2_with_bson_re = {'r': bson_re2}
         doc2_bson = (
-            b"\x12\x00\x00\x00"           # document length
-            b"\x0br\x00.*\x00ilmsux\x00"  # r: regex
+            b"\x11\x00\x00\x00"           # document length
+            b"\x0br\x00.*\x00imsux\x00"   # r: regex
             b"\x00")                      # document terminator
 
         self.assertEqual(doc2_bson, BSON.encode(doc2_with_re))
@@ -682,6 +768,9 @@ class TestBSON(unittest.TestCase):
         unicode_regex = re.compile('', re.U)
         self.assertEqual(re.U, Regex.from_native(unicode_regex).flags)
 
+    def test_regex_hash(self):
+        self.assertRaises(TypeError, hash, Regex('hello'))
+
     def test_exception_wrapping(self):
         # No matter what exception is raised while trying to decode BSON,
         # the final exception always matches InvalidBSON.
@@ -738,6 +827,11 @@ class TestBSON(unittest.TestCase):
         self.assertTrue(MaxKey() != MinKey())
         self.assertFalse(MaxKey() == MinKey())
 
+    def test_minkey_maxkey_hash(self):
+        self.assertEqual(hash(MaxKey()), hash(MaxKey()))
+        self.assertEqual(hash(MinKey()), hash(MinKey()))
+        self.assertNotEqual(hash(MaxKey()), hash(MinKey()))
+
     def test_timestamp_comparison(self):
         # Timestamp is initialized with time, inc. Time is the more
         # significant comparand.
@@ -784,9 +878,17 @@ class TestCodecOptions(unittest.TestCase):
         self.assertRaises(ValueError, CodecOptions, uuid_representation=7)
         self.assertRaises(ValueError, CodecOptions, uuid_representation=2)
 
+    def test_tzinfo(self):
+        self.assertRaises(TypeError, CodecOptions, tzinfo='pacific')
+        tz = FixedOffset(42, 'forty-two')
+        self.assertRaises(ValueError, CodecOptions, tzinfo=tz)
+        self.assertEqual(tz, CodecOptions(tz_aware=True, tzinfo=tz).tzinfo)
+
     def test_codec_options_repr(self):
-        r = ('CodecOptions(document_class=dict, tz_aware=False, '
-             'uuid_representation=PYTHON_LEGACY)')
+        r = ("CodecOptions(document_class=dict, tz_aware=False, "
+             "uuid_representation=PYTHON_LEGACY, "
+             "unicode_decode_error_handler='strict', "
+             "tzinfo=None)")
         self.assertEqual(r, repr(CodecOptions()))
 
     def test_decode_all_defaults(self):
@@ -803,6 +905,71 @@ class TestCodecOptions(unittest.TestCase):
         self.assertEqual(decoded['uuid'], doc['uuid'])
         self.assertIsNone(decoded['dt'].tzinfo)
 
+    def test_unicode_decode_error_handler(self):
+        enc = BSON.encode({"keystr": "foobar"})
+
+        # Test handling of bad key value.
+        invalid_key = BSON(enc[:7] + b'\xe9' + enc[8:])
+        replaced_key = b'ke\xef\xbf\xbdstr'.decode('utf-8')
+
+        dec = BSON.decode(invalid_key, CodecOptions(
+            unicode_decode_error_handler="replace"))
+        self.assertEqual(dec, {replaced_key: u("foobar")})
+
+        dec = BSON.decode(invalid_key, CodecOptions(
+            unicode_decode_error_handler="ignore"))
+        self.assertEqual(dec, {"kestr": "foobar"})
+
+        self.assertRaises(InvalidBSON, BSON.decode, invalid_key, CodecOptions(
+            unicode_decode_error_handler="strict"))
+        self.assertRaises(InvalidBSON, BSON.decode, invalid_key,
+                          CodecOptions())
+        self.assertRaises(InvalidBSON, BSON.decode, invalid_key)
+
+        # Test handing of bad string value.
+        invalid_val = BSON(enc[:18] + b'\xe9' + enc[19:])
+        replaced_val = b'fo\xef\xbf\xbdbar'.decode('utf-8')
+
+        dec = BSON.decode(invalid_val, CodecOptions(
+            unicode_decode_error_handler="replace"))
+        self.assertEqual(dec, {"keystr": replaced_val})
+
+        dec = BSON.decode(invalid_val, CodecOptions(
+            unicode_decode_error_handler="ignore"))
+        self.assertEqual(dec, {"keystr": "fobar"})
+
+        self.assertRaises(InvalidBSON, BSON.decode, invalid_val, CodecOptions(
+            unicode_decode_error_handler="strict"))
+        self.assertRaises(InvalidBSON, BSON.decode, invalid_val,
+                          CodecOptions())
+        self.assertRaises(InvalidBSON, BSON.decode, invalid_val)
+
+        # Test handing bad key + bad value.
+        invalid_both = BSON(
+            enc[:7] + b'\xe9' + enc[8:18] + b'\xe9' + enc[19:])
+
+        dec = BSON.decode(invalid_both, CodecOptions(
+            unicode_decode_error_handler="replace"))
+        self.assertEqual(dec, {replaced_key: replaced_val})
+
+        dec = BSON.decode(invalid_both, CodecOptions(
+            unicode_decode_error_handler="ignore"))
+        self.assertEqual(dec, {"kestr": "fobar"})
+
+        self.assertRaises(InvalidBSON, BSON.decode, invalid_both, CodecOptions(
+            unicode_decode_error_handler="strict"))
+        self.assertRaises(InvalidBSON, BSON.decode, invalid_both,
+                          CodecOptions())
+        self.assertRaises(InvalidBSON, BSON.decode, invalid_both)
+
+        # Test handling bad error mode.
+        dec = BSON.decode(enc, CodecOptions(
+            unicode_decode_error_handler="junk"))
+        self.assertEqual(dec, {"keystr": "foobar"})
+
+        self.assertRaises(InvalidBSON, BSON.decode, invalid_both,
+                          CodecOptions(unicode_decode_error_handler="junk"))
+
 
 if __name__ == "__main__":
     unittest.main()
