@@ -42,6 +42,7 @@ from pymongo.errors import (AutoReconnect,
                             CursorNotFound,
                             NetworkTimeout,
                             InvalidURI)
+from pymongo.message import _CursorAddress
 from pymongo.mongo_client import MongoClient
 from pymongo.pool import SocketInfo
 from pymongo.read_preferences import ReadPreference
@@ -205,19 +206,23 @@ class TestClient(IntegrationTest):
 
     def test_init_disconnected(self):
         c = rs_or_single_client(connect=False)
-
+        # is_primary causes client to block until connected
         self.assertIsInstance(c.is_primary, bool)
+
+        c = rs_or_single_client(connect=False)
         self.assertIsInstance(c.is_mongos, bool)
+        c = rs_or_single_client(connect=False)
         self.assertIsInstance(c.max_pool_size, int)
         self.assertIsInstance(c.nodes, frozenset)
 
+        c = rs_or_single_client(connect=False)
         self.assertEqual(c.codec_options, CodecOptions())
         self.assertIsInstance(c.max_bson_size, int)
-        self.assertIsInstance(c.max_write_batch_size, int)
+        c = rs_or_single_client(connect=False)
         self.assertFalse(c.primary)
         self.assertFalse(c.secondaries)
-
-        c.pymongo_test.command('ismaster')  # Auto-connect.
+        c = rs_or_single_client(connect=False)
+        self.assertIsInstance(c.max_write_batch_size, int)
 
         if client_context.is_rs:
             # The primary's host and port are from the replica set config.
@@ -707,7 +712,7 @@ class TestClient(IntegrationTest):
         new_sock_info = next(iter(pool.sockets))
         self.assertEqual(old_sock_info, new_sock_info)
 
-    def test_kill_cursors(self):
+    def test_kill_cursors_with_cursoraddress(self):
         if (client_context.is_mongos
                 and not client_context.version.at_least(2, 4, 7)):
             # Old mongos sends incorrectly formatted error response when
@@ -720,7 +725,40 @@ class TestClient(IntegrationTest):
         self.collection.insert_many([{'_id': i} for i in range(200)])
         cursor = self.collection.find().batch_size(1)
         next(cursor)
-        self.client.kill_cursors([cursor.cursor_id])
+        self.client.kill_cursors(
+            [cursor.cursor_id],
+            _CursorAddress(self.client.address, self.collection.full_name))
+
+        # Prevent killcursors from reaching the server while a getmore is in
+        # progress -- the server logs "Assertion: 16089:Cannot kill active
+        # cursor."
+        time.sleep(2)
+
+        def raises_cursor_not_found():
+            try:
+                next(cursor)
+                return False
+            except CursorNotFound:
+                return True
+
+        wait_until(raises_cursor_not_found, 'close cursor')
+
+    def test_kill_cursors_with_tuple(self):
+        if (client_context.is_mongos
+                and not client_context.version.at_least(2, 4, 7)):
+            # Old mongos sends incorrectly formatted error response when
+            # cursor isn't found, see SERVER-9738.
+            raise SkipTest("Can't test kill_cursors against old mongos")
+
+        self.collection = self.client.pymongo_test.test
+        self.collection.drop()
+
+        self.collection.insert_many([{'_id': i} for i in range(200)])
+        cursor = self.collection.find().batch_size(1)
+        next(cursor)
+        self.client.kill_cursors(
+            [cursor.cursor_id],
+            self.client.address)
 
         # Prevent killcursors from reaching the server while a getmore is in
         # progress -- the server logs "Assertion: 16089:Cannot kill active
@@ -837,7 +875,8 @@ class TestClient(IntegrationTest):
                                  serverSelectionTimeoutMS=100,
                                  replicaSet=client_context.replica_set_name)
             client._send_message_with_response(
-                operation=message._GetMore('collection', 101, 1234),
+                operation=message._GetMore('pymongo_test', 'collection',
+                                           101, 1234, client.codec_options),
                 address=('not-a-member', 27017))
 
 
@@ -1023,14 +1062,9 @@ class TestClientLazyConnect(IntegrationTest):
         lazy_client_trial(reset, find_one, test, self._get_client)
 
     def test_max_bson_size(self):
-        # Client should have sane defaults before connecting, and should update
-        # its configuration once connected.
         c = self._get_client()
-        self.assertEqual(16 * (1024 ** 2), c.max_bson_size)
-        self.assertEqual(2 * c.max_bson_size, c.max_message_size)
 
-        # Make the client connect, so that it sets its max_bson_size and
-        # max_message_size attributes.
+        # max_bson_size will cause the client to connect.
         ismaster = c.db.command('ismaster')
         self.assertEqual(ismaster['maxBsonObjectSize'], c.max_bson_size)
         if 'maxMessageSizeBytes' in ismaster:
