@@ -24,14 +24,19 @@ from collections import defaultdict
 
 sys.path[0:0] = [""]
 
+import bson
+
+from bson.raw_bson import RawBSONDocument
 from bson.regex import Regex
 from bson.code import Code
+from bson.codec_options import CodecOptions
 from bson.objectid import ObjectId
 from bson.py3compat import u, itervalues
 from bson.son import SON
 from pymongo import (ASCENDING, DESCENDING, GEO2D,
                      GEOHAYSTACK, GEOSPHERE, HASHED, TEXT)
-from pymongo import MongoClient
+from pymongo import MongoClient, monitoring
+from pymongo.bulk import BulkWriteError
 from pymongo.collection import Collection, ReturnDocument
 from pymongo.command_cursor import CommandCursor
 from pymongo.cursor import CursorType
@@ -40,9 +45,10 @@ from pymongo.errors import (DocumentTooLarge,
                             InvalidDocument,
                             InvalidName,
                             InvalidOperation,
-                            OperationFailure)
-from pymongo.message import _COMMAND_OVERHEAD
-from pymongo.operations import IndexModel
+                            OperationFailure,
+                            WriteConcernError)
+from pymongo.message import _COMMAND_OVERHEAD, _gen_find_command
+from pymongo.operations import *
 from pymongo.read_preferences import ReadPreference
 from pymongo.results import (InsertOneResult,
                              InsertManyResult,
@@ -51,7 +57,8 @@ from pymongo.results import (InsertOneResult,
 from pymongo.write_concern import WriteConcern
 from test.test_client import IntegrationTest
 from test.utils import (is_mongos, enable_text_search, get_pool,
-                        rs_or_single_client, wait_until)
+                        rs_or_single_client, single_client,
+                        wait_until, EventListener)
 from test import client_context, host, port, unittest
 
 
@@ -480,6 +487,83 @@ class TestCollection(IntegrationTest):
         # Index wasn't created, only the default index on _id
         self.assertEqual(1, len(db.test.index_information()))
 
+    # Get the plan dynamically because the explain format will change.
+    def get_plan_stage(self, root, stage):
+        if root.get('stage') == stage:
+            return root
+        elif "inputStage" in root:
+            return self.get_plan_stage(root['inputStage'], stage)
+        elif "inputStages" in root:
+            for i in root['inputStages']:
+                stage = self.get_plan_stage(i, stage)
+                if stage:
+                    return stage
+        elif "shards" in root:
+            for i in root['shards']:
+                stage = self.get_plan_stage(i['winningPlan'], stage)
+                if stage:
+                    return stage
+        return {}
+
+    @client_context.require_version_min(3, 1, 9, -1)
+    def test_index_filter(self):
+        db = self.db
+        db.drop_collection("test")
+
+        # Test bad filter spec on create.
+        self.assertRaises(OperationFailure, db.test.create_index, "x",
+                          partialFilterExpression=5)
+        self.assertRaises(OperationFailure, db.test.create_index, "x",
+                          partialFilterExpression={"x": {"$asdasd": 3}})
+        self.assertRaises(OperationFailure, db.test.create_index, "x",
+                          partialFilterExpression={"$and": 5})
+        self.assertRaises(OperationFailure, db.test.create_index, "x",
+                          partialFilterExpression={
+                              "$and": [{"$and":  [{"x": {"$lt": 2}},
+                                                  {"x": {"$gt": 0}}]},
+                                       {"x": {"$exists": True}}]})
+
+        self.assertEqual("x_1", db.test.create_index(
+            [('x', ASCENDING)], partialFilterExpression={"a": {"$lte": 1.5}}))
+        db.test.insert_one({"x": 5, "a": 2})
+        db.test.insert_one({"x": 6, "a": 1})
+
+        # Operations that use the partial index.
+        explain = db.test.find({"x": 6, "a": 1}).explain()
+        stage = self.get_plan_stage(explain['queryPlanner']['winningPlan'],
+                                    'IXSCAN')
+        self.assertEqual("x_1", stage.get('indexName'))
+        self.assertTrue(stage.get('isPartial'))
+
+        explain = db.test.find({"x": {"$gt": 1}, "a": 1}).explain()
+        stage = self.get_plan_stage(explain['queryPlanner']['winningPlan'],
+                                    'IXSCAN')
+        self.assertEqual("x_1", stage.get('indexName'))
+        self.assertTrue(stage.get('isPartial'))
+
+        explain = db.test.find({"x": 6, "a": {"$lte": 1}}).explain()
+        stage = self.get_plan_stage(explain['queryPlanner']['winningPlan'],
+                                    'IXSCAN')
+        self.assertEqual("x_1", stage.get('indexName'))
+        self.assertTrue(stage.get('isPartial'))
+
+        # Operations that do not use the partial index.
+        explain = db.test.find({"x": 6, "a": {"$lte": 1.6}}).explain()
+        stage = self.get_plan_stage(explain['queryPlanner']['winningPlan'],
+                                    'COLLSCAN')
+        self.assertNotEqual({}, stage)
+        explain = db.test.find({"x": 6}).explain()
+        stage = self.get_plan_stage(explain['queryPlanner']['winningPlan'],
+                                    'COLLSCAN')
+        self.assertNotEqual({}, stage)
+
+        # Test drop_indexes.
+        db.test.drop_index("x_1")
+        explain = db.test.find({"x": 6, "a": 1}).explain()
+        stage = self.get_plan_stage(explain['queryPlanner']['winningPlan'],
+                                    'COLLSCAN')
+        self.assertNotEqual({}, stage)
+
     def test_field_selection(self):
         db = self.db
         db.drop_collection("test")
@@ -576,6 +660,12 @@ class TestCollection(IntegrationTest):
         # The insert failed duplicate key...
         wait_until(lambda: 2 == db.test.count(), 'forcing duplicate key error')
 
+        document = RawBSONDocument(
+            bson.BSON.encode({'_id': ObjectId(), 'foo': 'bar'}))
+        result = db.test.insert_one(document)
+        self.assertTrue(isinstance(result, InsertOneResult))
+        self.assertEqual(result.inserted_id, None)
+
     def test_insert_many(self):
         db = self.db
         db.test.drop()
@@ -604,13 +694,20 @@ class TestCollection(IntegrationTest):
             self.assertEqual(1, db.test.count({"_id": _id}))
         self.assertTrue(result.acknowledged)
 
+        docs = [RawBSONDocument(bson.BSON.encode({"_id": i + 5}))
+                for i in range(5)]
+        result = db.test.insert_many(docs)
+        self.assertTrue(isinstance(result, InsertManyResult))
+        self.assertTrue(isinstance(result.inserted_ids, list))
+        self.assertEqual([], result.inserted_ids)
+
         db = db.client.get_database(db.name,
                                     write_concern=WriteConcern(w=0))
         docs = [{} for _ in range(5)]
         result = db.test.insert_many(docs)
         self.assertTrue(isinstance(result, InsertManyResult))
         self.assertFalse(result.acknowledged)
-        self.assertEqual(15, db.test.count())
+        self.assertEqual(20, db.test.count())
 
     def test_delete_one(self):
         self.db.test.drop()
@@ -672,6 +769,205 @@ class TestCollection(IntegrationTest):
         self.assertRaises(
             DocumentTooLarge, coll.delete_one, {'data': large})
 
+    @client_context.require_version_min(3, 1, 9, -1)
+    @client_context.require_no_auth
+    def test_insert_bypass_document_validation(self):
+        db = self.db
+        db.test.drop()
+        db.create_collection("test", validator={"a": {"$exists": True}})
+        db_w0 = self.db.client.get_database(
+            self.db.name, write_concern=WriteConcern(w=0))
+
+        # Test insert_one
+        self.assertRaises(OperationFailure, db.test.insert_one,
+                          {"_id": 1, "x": 100})
+        result = db.test.insert_one({"_id": 1, "x": 100},
+                                    bypass_document_validation=True)
+        self.assertTrue(isinstance(result, InsertOneResult))
+        self.assertEqual(1, result.inserted_id)
+        result = db.test.insert_one({"_id":2, "a":0})
+        self.assertTrue(isinstance(result, InsertOneResult))
+        self.assertEqual(2, result.inserted_id)
+
+        self.assertRaises(OperationFailure, db_w0.test.insert_one,
+                          {"x": 1}, bypass_document_validation=True)
+
+        # Test insert_many
+        docs = [{"_id": i, "x": 100 - i} for i in range(3, 100)]
+        self.assertRaises(OperationFailure, db.test.insert_many, docs)
+        result = db.test.insert_many(docs, bypass_document_validation=True)
+        self.assertTrue(isinstance(result, InsertManyResult))
+        self.assertTrue(97, len(result.inserted_ids))
+        for doc in docs:
+            _id = doc["_id"]
+            self.assertTrue(isinstance(_id, int))
+            self.assertTrue(_id in result.inserted_ids)
+            self.assertEqual(1, db.test.count({"x": doc["x"]}))
+        self.assertTrue(result.acknowledged)
+        docs = [{"_id": i, "a": 200 - i} for i in range(100, 200)]
+        result = db.test.insert_many(docs)
+        self.assertTrue(isinstance(result, InsertManyResult))
+        self.assertTrue(97, len(result.inserted_ids))
+        for doc in docs:
+            _id = doc["_id"]
+            self.assertTrue(isinstance(_id, int))
+            self.assertTrue(_id in result.inserted_ids)
+            self.assertEqual(1, db.test.count({"a": doc["a"]}))
+        self.assertTrue(result.acknowledged)
+
+        self.assertRaises(OperationFailure, db_w0.test.insert_many,
+                          [{"x": 1}, {"x": 2}],
+                          bypass_document_validation=True)
+
+    @client_context.require_version_min(3, 1, 9, -1)
+    @client_context.require_no_auth
+    def test_replace_bypass_document_validation(self):
+        db = self.db
+        db.test.drop()
+        db.create_collection("test", validator={"a": {"$exists": True}})
+        db_w0 = self.db.client.get_database(
+            self.db.name, write_concern=WriteConcern(w=0))
+
+        # Test replace_one
+        db.test.insert_one({"a": 101})
+        self.assertRaises(OperationFailure, db.test.replace_one,
+                          {"a": 101}, {"y": 1})
+        self.assertEqual(0, db.test.count({"y": 1}))
+        self.assertEqual(1, db.test.count({"a": 101}))
+        db.test.replace_one({"a": 101}, {"y": 1},
+                            bypass_document_validation=True)
+        self.assertEqual(0, db.test.count({"a": 101}))
+        self.assertEqual(1, db.test.count({"y": 1}))
+        db.test.replace_one({"y": 1}, {"a": 102})
+        self.assertEqual(0, db.test.count({"y": 1}))
+        self.assertEqual(0, db.test.count({"a": 101}))
+        self.assertEqual(1, db.test.count({"a": 102}))
+
+        db.test.insert_one({"y": 1}, bypass_document_validation=True)
+        self.assertRaises(OperationFailure, db.test.replace_one,
+                          {"y": 1}, {"x": 101})
+        self.assertEqual(0, db.test.count({"x": 101}))
+        self.assertEqual(1, db.test.count({"y": 1}))
+        db.test.replace_one({"y": 1}, {"x": 101},
+                            bypass_document_validation=True)
+        self.assertEqual(0, db.test.count({"y": 1}))
+        self.assertEqual(1, db.test.count({"x": 101}))
+        db.test.replace_one({"x": 101}, {"a": 103},
+                            bypass_document_validation=False)
+        self.assertEqual(0, db.test.count({"x": 101}))
+        self.assertEqual(1, db.test.count({"a": 103}))
+
+        self.assertRaises(OperationFailure, db_w0.test.replace_one, {"y": 1},
+                          {"x": 1}, bypass_document_validation=True)
+
+    @client_context.require_version_min(3, 1, 9, -1)
+    @client_context.require_no_auth
+    def test_update_bypass_document_validation(self):
+        db = self.db
+        db.test.drop()
+        db.test.insert_one({"z": 5})
+        db.command(SON([("collMod", "test"),
+                        ("validator", {"z": {"$gte": 0}})]))
+        db_w0 = self.db.client.get_database(
+            self.db.name, write_concern=WriteConcern(w=0))
+
+        # Test update_one
+        self.assertRaises(OperationFailure, db.test.update_one,
+                          {"z": 5}, {"$inc": {"z": -10}})
+        self.assertEqual(0, db.test.count({"z": -5}))
+        self.assertEqual(1, db.test.count({"z": 5}))
+        db.test.update_one({"z": 5}, {"$inc": {"z": -10}},
+                           bypass_document_validation=True)
+        self.assertEqual(0, db.test.count({"z": 5}))
+        self.assertEqual(1, db.test.count({"z": -5}))
+        db.test.update_one({"z": -5}, {"$inc": {"z": 6}},
+                           bypass_document_validation=False)
+        self.assertEqual(1, db.test.count({"z": 1}))
+        self.assertEqual(0, db.test.count({"z": -5}))
+
+        db.test.insert_one({"z": -10},
+                           bypass_document_validation=True)
+        self.assertRaises(OperationFailure, db.test.update_one,
+                          {"z": -10}, {"$inc": {"z": 1}})
+        self.assertEqual(0, db.test.count({"z": -9}))
+        self.assertEqual(1, db.test.count({"z": -10}))
+        db.test.update_one({"z": -10}, {"$inc": {"z": 1}},
+                           bypass_document_validation=True)
+        self.assertEqual(1, db.test.count({"z": -9}))
+        self.assertEqual(0, db.test.count({"z": -10}))
+        db.test.update_one({"z": -9}, {"$inc": {"z": 9}},
+                           bypass_document_validation=False)
+        self.assertEqual(0, db.test.count({"z": -9}))
+        self.assertEqual(1, db.test.count({"z": 0}))
+
+        self.assertRaises(OperationFailure, db_w0.test.update_one, {"y": 1},
+                          {"$inc": {"x": 1}}, bypass_document_validation=True)
+
+        # Test update_many
+        db.test.insert_many([{"z": i} for i in range(3, 101)])
+        db.test.insert_one({"y": 0},
+                           bypass_document_validation=True)
+        self.assertRaises(OperationFailure, db.test.update_many, {},
+                          {"$inc": {"z": -100}})
+        self.assertEqual(100, db.test.count({"z": {"$gte": 0}}))
+        self.assertEqual(0, db.test.count({"z": {"$lt": 0}}))
+        self.assertEqual(0, db.test.count({"y": 0, "z": -100}))
+        db.test.update_many({"z": {"$gte": 0}}, {"$inc": {"z": -100}},
+                            bypass_document_validation=True)
+        self.assertEqual(0, db.test.count({"z": {"$gt": 0}}))
+        self.assertEqual(100, db.test.count({"z": {"$lte": 0}}))
+        db.test.update_many({"z": {"$gt": -50}}, {"$inc": {"z": 100}},
+                            bypass_document_validation=False)
+        self.assertEqual(50, db.test.count({"z": {"$gt": 0}}))
+        self.assertEqual(50, db.test.count({"z": {"$lt": 0}}))
+
+        db.test.insert_many([{"z": -i} for i in range(50)],
+                            bypass_document_validation=True)
+        self.assertRaises(OperationFailure, db.test.update_many,
+            {}, {"$inc": {"z": 1}})
+        self.assertEqual(100, db.test.count({"z": {"$lte": 0}}))
+        self.assertEqual(50, db.test.count({"z": {"$gt": 1}}))
+        db.test.update_many({"z": {"$gte": 0}}, {"$inc": {"z": -100}},
+                            bypass_document_validation=True)
+        self.assertEqual(0, db.test.count({"z": {"$gt": 0}}))
+        self.assertEqual(150, db.test.count({"z": {"$lte": 0}}))
+        db.test.update_many({"z": {"$lte": 0}}, {"$inc": {"z": 100}},
+                            bypass_document_validation=False)
+        self.assertEqual(150, db.test.count({"z": {"$gte": 0}}))
+        self.assertEqual(0, db.test.count({"z": {"$lt": 0}}))
+
+        self.assertRaises(OperationFailure, db_w0.test.update_many, {"y": 1},
+                          {"$inc": {"x": 1}}, bypass_document_validation=True)
+
+    @client_context.require_version_min(3, 1, 9, -1)
+    @client_context.require_no_auth
+    def test_bypass_document_validation_bulk_write(self):
+        db = self.db
+        db.test.drop()
+        db.create_collection("test", validator={"a": {"$gte": 0}})
+        db_w0 = self.db.client.get_database(
+            self.db.name, write_concern=WriteConcern(w=0))
+
+        ops = [InsertOne({"a": -10}),
+               InsertOne({"a": -11}),
+               InsertOne({"a": -12}),
+               UpdateOne({"a": {"$lte": -10}}, {"$inc": {"a": 1}}),
+               UpdateMany({"a": {"$lte": -10}}, {"$inc": {"a": 1}}),
+               ReplaceOne({"a": {"$lte": -10}}, {"a": -1})]
+        db.test.bulk_write(ops, bypass_document_validation=True)
+
+        self.assertEqual(3, db.test.count())
+        self.assertEqual(1, db.test.count({"a": -11}))
+        self.assertEqual(1, db.test.count({"a": -1}))
+        self.assertEqual(1, db.test.count({"a": -9}))
+
+        # Assert that the operations would fail without bypass_doc_val
+        for op in ops:
+            self.assertRaises(BulkWriteError, db.test.bulk_write, [op])
+
+        self.assertRaises(OperationFailure, db_w0.test.bulk_write, ops,
+                          bypass_document_validation=True)
+
     def test_find_by_default_dct(self):
         db = self.db
         db.test.insert_one({'foo': 'bar'})
@@ -881,6 +1177,17 @@ class TestCollection(IntegrationTest):
         self.assertEqual(0, db.test.count({"x": 1}))
         self.assertEqual(db.test.find_one(id1)["y"], 1)
 
+        replacement = RawBSONDocument(bson.BSON.encode({"_id": id1, "z": 1}))
+        result = db.test.replace_one({"y": 1}, replacement, True)
+        self.assertTrue(isinstance(result, UpdateResult))
+        self.assertEqual(1, result.matched_count)
+        self.assertTrue(result.modified_count in (None, 1))
+        self.assertIsNone(result.upserted_id)
+        self.assertTrue(result.acknowledged)
+        self.assertEqual(1, db.test.count({"z": 1}))
+        self.assertEqual(0, db.test.count({"y": 1}))
+        self.assertEqual(db.test.find_one(id1)["z"], 1)
+
         result = db.test.replace_one({"x": 2}, {"y": 2}, True)
         self.assertTrue(isinstance(result, UpdateResult))
         self.assertEqual(0, result.matched_count)
@@ -1075,6 +1382,23 @@ class TestCollection(IntegrationTest):
         self.assertTrue(isinstance(result, CommandCursor))
         self.assertEqual([{'foo': [1, 2]}], list(result))
 
+    def test_aggregate_raw_bson(self):
+        db = self.db
+        db.drop_collection("test")
+        db.test.insert_one({'foo': [1, 2]})
+
+        self.assertRaises(TypeError, db.test.aggregate, "wow")
+
+        pipeline = {"$project": {"_id": False, "foo": True}}
+        result = db.get_collection(
+            'test',
+            codec_options=CodecOptions(document_class=RawBSONDocument)
+        ).aggregate([pipeline], useCursor=False)
+        self.assertTrue(isinstance(result, CommandCursor))
+        first_result = next(result)
+        self.assertIsInstance(first_result, RawBSONDocument)
+        self.assertEqual([1, 2], list(first_result['foo']))
+
     @client_context.require_version_min(2, 5, 1)
     def test_aggregation_cursor_validation(self):
         db = self.db
@@ -1668,6 +1992,104 @@ class TestCollection(IntegrationTest):
                                                   {'$inc': {'i': 1}},
                                                   sort=sort)['j'])
 
+    def test_find_one_and_write_concern(self):
+        listener = EventListener()
+        saved_listeners = monitoring._LISTENERS
+        monitoring._LISTENERS = monitoring._Listeners([])
+        db = single_client(event_listeners=[listener])[self.db.name]
+        # non-default WriteConcern.
+        c_w0 = db.get_collection(
+            'test', write_concern=WriteConcern(w=0))
+        # default WriteConcern.
+        c_default = db.get_collection('test', write_concern=WriteConcern())
+        results = listener.results
+        # Authenticate the client and throw out auth commands from the listener.
+        db.command('ismaster')
+        results.clear()
+        try:
+            if client_context.version.at_least(3, 1, 9, -1):
+                c_w0.find_and_modify(
+                    {'_id': 1}, {'$set': {'foo': 'bar'}})
+                self.assertEqual(
+                    {'w': 0}, results['started'][0].command['writeConcern'])
+                results.clear()
+
+                c_w0.find_one_and_update(
+                    {'_id': 1}, {'$set': {'foo': 'bar'}})
+                self.assertEqual(
+                    {'w': 0}, results['started'][0].command['writeConcern'])
+                results.clear()
+
+                c_w0.find_one_and_replace({'_id': 1}, {'foo': 'bar'})
+                self.assertEqual(
+                    {'w': 0}, results['started'][0].command['writeConcern'])
+                results.clear()
+
+                c_w0.find_one_and_delete({'_id': 1})
+                self.assertEqual(
+                    {'w': 0}, results['started'][0].command['writeConcern'])
+                results.clear()
+
+                # Test write concern errors.
+                if client_context.is_rs:
+                    c_wc_error = db.get_collection(
+                        'test',
+                        write_concern=WriteConcern(
+                            w=len(client_context.nodes) + 1))
+                    self.assertRaises(
+                        WriteConcernError,
+                        c_wc_error.find_and_modify,
+                        {'_id': 1}, {'$set': {'foo': 'bar'}})
+                    self.assertRaises(
+                        WriteConcernError,
+                        c_wc_error.find_one_and_update,
+                        {'_id': 1}, {'$set': {'foo': 'bar'}})
+                    self.assertRaises(
+                        WriteConcernError,
+                        c_wc_error.find_one_and_replace,
+                        {'w': 0}, results['started'][0].command['writeConcern'])
+                    self.assertRaises(
+                        WriteConcernError,
+                        c_wc_error.find_one_and_delete,
+                        {'w': 0}, results['started'][0].command['writeConcern'])
+                    results.clear()
+            else:
+                c_w0.find_and_modify(
+                    {'_id': 1}, {'$set': {'foo': 'bar'}})
+                self.assertNotIn('writeConcern', results['started'][0].command)
+                results.clear()
+
+                c_w0.find_one_and_update(
+                    {'_id': 1}, {'$set': {'foo': 'bar'}})
+                self.assertNotIn('writeConcern', results['started'][0].command)
+                results.clear()
+
+                c_w0.find_one_and_replace({'_id': 1}, {'foo': 'bar'})
+                self.assertNotIn('writeConcern', results['started'][0].command)
+                results.clear()
+
+                c_w0.find_one_and_delete({'_id': 1})
+                self.assertNotIn('writeConcern', results['started'][0].command)
+                results.clear()
+
+            c_default.find_and_modify({'_id': 1}, {'$set': {'foo': 'bar'}})
+            self.assertNotIn('writeConcern', results['started'][0].command)
+            results.clear()
+
+            c_default.find_one_and_update({'_id': 1}, {'$set': {'foo': 'bar'}})
+            self.assertNotIn('writeConcern', results['started'][0].command)
+            results.clear()
+
+            c_default.find_one_and_replace({'_id': 1}, {'foo': 'bar'})
+            self.assertNotIn('writeConcern', results['started'][0].command)
+            results.clear()
+
+            c_default.find_one_and_delete({'_id': 1})
+            self.assertNotIn('writeConcern', results['started'][0].command)
+            results.clear()
+        finally:
+            monitoring._LISTENERS = saved_listeners
+
     def test_find_with_nested(self):
         c = self.db.test
         c.drop()
@@ -1725,6 +2147,15 @@ class TestCollection(IntegrationTest):
         for doc in c.find():
             self.assertTrue(isinstance(doc['r'], Regex))
 
+    def test_find_command_generation(self):
+        cmd = _gen_find_command(
+            'coll', {'$query': {'foo': 1}, '$dumb': 2}, None, 0, 0, 0, None)
+        self.assertEqual(
+            cmd.to_dict(),
+            SON([('find', 'coll'),
+                 ('$dumb', 2),
+                 ('filter', {'foo': 1})]).to_dict())
+
 
 if __name__ == "__main__":
     unittest.main()
