@@ -19,8 +19,6 @@ import os
 import re
 import sys
 
-from collections import defaultdict
-
 sys.path[0:0] = [""]
 
 import pymongo
@@ -31,7 +29,7 @@ from pymongo.errors import OperationFailure
 from pymongo.read_preferences import make_read_preference
 from pymongo.write_concern import WriteConcern
 from test import unittest, client_context
-from test.utils import single_client, wait_until
+from test.utils import single_client, wait_until, EventListener
 
 # Location of JSON test specifications.
 _TEST_PATH = os.path.join(
@@ -45,21 +43,6 @@ def camel_to_snake(camel):
     return re.sub('([a-z0-9])([A-Z])', r'\1_\2', snake).lower()
 
 
-class EventListener(monitoring.CommandListener):
-
-    def __init__(self):
-        self.results = defaultdict(list)
-
-    def started(self, event):
-        self.results['started'].append(event)
-
-    def succeeded(self, event):
-        self.results['succeeded'].append(event)
-
-    def failed(self, event):
-        self.results['failed'].append(event)
-
-
 class TestAllScenarios(unittest.TestCase):
 
     @classmethod
@@ -118,6 +101,10 @@ def create_test(scenario_def):
                 except OperationFailure:
                     pass
             elif name == 'find':
+                # XXX: Skip killCursors test when using the find command.
+                if (client_context.version.at_least(3, 1, 1) and
+                        'limit' in args):
+                    continue
                 if 'sort' in args:
                     args['sort'] = list(args['sort'].items())
                 try:
