@@ -25,6 +25,7 @@ from bson.code import Code
 from bson.py3compat import u, PY3
 from bson.son import SON
 from pymongo import (MongoClient,
+                     monitoring,
                      ASCENDING,
                      DESCENDING,
                      ALL,
@@ -41,7 +42,7 @@ from test import (client_context,
                   host,
                   port,
                   IntegrationTest)
-from test.utils import server_started_with_auth
+from test.utils import server_started_with_auth, single_client, EventListener
 
 if PY3:
     long = int
@@ -121,17 +122,6 @@ class TestCursorNoConnect(unittest.TestCase):
         cursor.remove_option(32)
         self.assertEqual(2, cursor._Cursor__query_flags)
 
-        # Exhaust - which mongos doesn't support
-        cursor = self.db.test.find(cursor_type=CursorType.EXHAUST)
-        self.assertEqual(64, cursor._Cursor__query_flags)
-        cursor2 = self.db.test.find().add_option(64)
-        self.assertEqual(cursor._Cursor__query_flags,
-                         cursor2._Cursor__query_flags)
-        self.assertTrue(cursor._Cursor__exhaust)
-        cursor.remove_option(64)
-        self.assertEqual(0, cursor._Cursor__query_flags)
-        self.assertFalse(cursor._Cursor__exhaust)
-
         # Partial
         cursor = self.db.test.find(allow_partial_results=True)
         self.assertEqual(128, cursor._Cursor__query_flags)
@@ -141,6 +131,18 @@ class TestCursorNoConnect(unittest.TestCase):
         cursor.remove_option(128)
         self.assertEqual(0, cursor._Cursor__query_flags)
 
+        # Exhaust - which mongos doesn't support
+        if not self.db.client.is_mongos:
+            cursor = self.db.test.find(cursor_type=CursorType.EXHAUST)
+            self.assertEqual(64, cursor._Cursor__query_flags)
+            cursor2 = self.db.test.find().add_option(64)
+            self.assertEqual(cursor._Cursor__query_flags,
+                             cursor2._Cursor__query_flags)
+            self.assertTrue(cursor._Cursor__exhaust)
+            cursor.remove_option(64)
+            self.assertEqual(0, cursor._Cursor__query_flags)
+            self.assertFalse(cursor._Cursor__exhaust)
+
 
 class TestCursor(IntegrationTest):
 
@@ -190,6 +192,114 @@ class TestCursor(IntegrationTest):
                                      "maxTimeAlwaysTimeOut",
                                      mode="off")
 
+    @client_context.require_version_min(3, 1, 9, -1)
+    def test_max_await_time_ms(self):
+        db = self.db
+        db.pymongo_test.drop()
+        coll = db.create_collection("pymongo_test", capped=True, size=4096)
+
+        self.assertRaises(TypeError, coll.find().max_await_time_ms, 'foo')
+        coll.insert_one({"amalia": 1})
+        coll.insert_one({"amalia": 2})
+
+        coll.find().max_await_time_ms(None)
+        coll.find().max_await_time_ms(long(1))
+
+        # When cursor is not tailable_await
+        cursor = coll.find()
+        self.assertEqual(None, cursor._Cursor__max_await_time_ms)
+        cursor = coll.find().max_await_time_ms(99)
+        self.assertEqual(None, cursor._Cursor__max_await_time_ms)
+
+        # If cursor is tailable_await and timeout is unset
+        cursor = coll.find(cursor_type=CursorType.TAILABLE_AWAIT)
+        self.assertEqual(None, cursor._Cursor__max_await_time_ms)
+
+        # If cursor is tailable_await and timeout is set
+        cursor = coll.find(
+            cursor_type=CursorType.TAILABLE_AWAIT).max_await_time_ms(99)
+        self.assertEqual(99, cursor._Cursor__max_await_time_ms)
+
+        cursor = coll.find(
+            cursor_type=CursorType.TAILABLE_AWAIT).max_await_time_ms(
+                10).max_await_time_ms(90)
+        self.assertEqual(90, cursor._Cursor__max_await_time_ms)
+
+        listener = EventListener()
+        saved_listeners = monitoring._LISTENERS
+        monitoring._LISTENERS = monitoring._Listeners([])
+        coll = single_client(
+            event_listeners=[listener])[self.db.name].pymongo_test
+        results = listener.results
+
+        try:
+            # Tailable_await defaults.
+            list(coll.find(cursor_type=CursorType.TAILABLE_AWAIT))
+            # find
+            self.assertFalse('maxTimeMS' in results['started'][0].command)
+            # getMore
+            self.assertFalse('maxTimeMS' in results['started'][1].command)
+            results.clear()
+
+            # Tailable_await with max_await_time_ms set.
+            list(coll.find(
+                cursor_type=CursorType.TAILABLE_AWAIT).max_await_time_ms(99))
+            # find
+            self.assertFalse('maxTimeMS' in results['started'][0].command)
+            # getMore
+            self.assertTrue('maxTimeMS' in results['started'][1].command)
+            self.assertEqual(99, results['started'][1].command['maxTimeMS'])
+            results.clear()
+
+            # Tailable_await with max_time_ms
+            list(coll.find(
+                cursor_type=CursorType.TAILABLE_AWAIT).max_time_ms(1))
+            # find
+            self.assertTrue('maxTimeMS' in results['started'][0].command)
+            self.assertEqual(1, results['started'][0].command['maxTimeMS'])
+            # getMore
+            self.assertFalse('maxTimeMS' in results['started'][1].command)
+            results.clear()
+
+            # Tailable_await with both max_time_ms and max_await_time_ms
+            list(coll.find(
+                cursor_type=CursorType.TAILABLE_AWAIT).max_time_ms(
+                    1).max_await_time_ms(99))
+            # find
+            self.assertTrue('maxTimeMS' in results['started'][0].command)
+            self.assertEqual(1, results['started'][0].command['maxTimeMS'])
+            # getMore
+            self.assertTrue('maxTimeMS' in results['started'][1].command)
+            self.assertEqual(99, results['started'][1].command['maxTimeMS'])
+            results.clear()
+
+            # Non tailable_await with max_await_time_ms
+            list(coll.find(batch_size=1).max_await_time_ms(99))
+            # find
+            self.assertFalse('maxTimeMS' in results['started'][0].command)
+            # getMore
+            self.assertFalse('maxTimeMS' in results['started'][1].command)
+            results.clear()
+
+            # Non tailable_await with max_time_ms
+            list(coll.find(batch_size=1).max_time_ms(99))
+            # find
+            self.assertTrue('maxTimeMS' in results['started'][0].command)
+            self.assertEqual(99, results['started'][0].command['maxTimeMS'])
+            # getMore
+            self.assertFalse('maxTimeMS' in results['started'][1].command)
+
+            # Non tailable_await with both max_time_ms and max_await_time_ms
+            list(coll.find(batch_size=1).max_time_ms(99).max_await_time_ms(88))
+            # find
+            self.assertTrue('maxTimeMS' in results['started'][0].command)
+            self.assertEqual(99, results['started'][0].command['maxTimeMS'])
+            # getMore
+            self.assertFalse('maxTimeMS' in results['started'][1].command)
+
+        finally:
+            monitoring._LISTENERS = saved_listeners
+
     @client_context.require_version_min(2, 5, 3, -1)
     @client_context.require_test_commands
     def test_max_time_ms_getmore(self):
