@@ -1,4 +1,4 @@
-# Copyright 2009-2014 MongoDB, Inc.
+# Copyright 2009-2015 MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,13 +15,13 @@
 """Tests for the dbref module."""
 
 import pickle
-import unittest
 import sys
 sys.path[0:0] = [""]
 
 from bson.dbref import DBRef
 from bson.objectid import ObjectId
-from bson.py3compat import b
+from bson.py3compat import u
+from test import unittest
 
 from copy import deepcopy
 
@@ -37,9 +37,9 @@ class TestDBRef(unittest.TestCase):
         self.assertRaises(TypeError, DBRef, None, a)
         self.assertRaises(TypeError, DBRef, "coll", a, 5)
         self.assertTrue(DBRef("coll", a))
-        self.assertTrue(DBRef(u"coll", a))
-        self.assertTrue(DBRef(u"coll", 5))
-        self.assertTrue(DBRef(u"coll", 5, "database"))
+        self.assertTrue(DBRef(u("coll"), a))
+        self.assertTrue(DBRef(u("coll"), 5))
+        self.assertTrue(DBRef(u("coll"), 5, "database"))
 
     def test_read_only(self):
         a = DBRef("coll", ObjectId())
@@ -60,10 +60,10 @@ class TestDBRef(unittest.TestCase):
         self.assertEqual(repr(DBRef("coll",
                                     ObjectId("1234567890abcdef12345678"))),
                          "DBRef('coll', ObjectId('1234567890abcdef12345678'))")
-        self.assertEqual(repr(DBRef(u"coll",
+        self.assertEqual(repr(DBRef(u("coll"),
                               ObjectId("1234567890abcdef12345678"))),
                          "DBRef(%s, ObjectId('1234567890abcdef12345678'))"
-                         % (repr(u'coll'),)
+                         % (repr(u('coll')),)
                         )
         self.assertEqual(repr(DBRef("coll", 5, foo="bar")),
                          "DBRef('coll', 5, foo='bar')")
@@ -76,23 +76,23 @@ class TestDBRef(unittest.TestCase):
         obj_id = ObjectId("1234567890abcdef12345678")
 
         self.assertEqual(DBRef('foo', 5), DBRef('foo', 5))
-        self.assertEqual(DBRef("coll", obj_id), DBRef(u"coll", obj_id))
+        self.assertEqual(DBRef("coll", obj_id), DBRef(u("coll"), obj_id))
         self.assertNotEqual(DBRef("coll", obj_id),
-                            DBRef(u"coll", obj_id, "foo"))
+                            DBRef(u("coll"), obj_id, "foo"))
         self.assertNotEqual(DBRef("coll", obj_id), DBRef("col", obj_id))
         self.assertNotEqual(DBRef("coll", obj_id),
-                            DBRef("coll", ObjectId(b("123456789011"))))
+                            DBRef("coll", ObjectId(b"123456789011")))
         self.assertNotEqual(DBRef("coll", obj_id), 4)
         self.assertEqual(DBRef("coll", obj_id, "foo"),
-                         DBRef(u"coll", obj_id, "foo"))
+                         DBRef(u("coll"), obj_id, "foo"))
         self.assertNotEqual(DBRef("coll", obj_id, "foo"),
-                            DBRef(u"coll", obj_id, "bar"))
+                            DBRef(u("coll"), obj_id, "bar"))
 
         # Explicitly test inequality
         self.assertFalse(DBRef('foo', 5) != DBRef('foo', 5))
-        self.assertFalse(DBRef("coll", obj_id) != DBRef(u"coll", obj_id))
+        self.assertFalse(DBRef("coll", obj_id) != DBRef(u("coll"), obj_id))
         self.assertFalse(DBRef("coll", obj_id, "foo") !=
-                         DBRef(u"coll", obj_id, "foo"))
+                         DBRef(u("coll"), obj_id, "foo"))
 
     def test_kwargs(self):
         self.assertEqual(DBRef("coll", 5, foo="bar"),
