@@ -17,8 +17,6 @@ import sys
 import time
 import warnings
 
-from collections import defaultdict
-
 sys.path[0:0] = [""]
 
 from bson.objectid import ObjectId
@@ -30,22 +28,7 @@ from pymongo.errors import NotMasterError, OperationFailure
 from pymongo.read_preferences import ReadPreference
 from pymongo.write_concern import WriteConcern
 from test import unittest, client_context, client_knobs
-from test.utils import single_client
-
-
-class EventListener(monitoring.CommandListener):
-
-    def __init__(self):
-        self.results = defaultdict(list)
-
-    def started(self, event):
-        self.results['started'].append(event)
-
-    def succeeded(self, event):
-        self.results['succeeded'].append(event)
-
-    def failed(self, event):
-        self.results['failed'].append(event)
+from test.utils import single_client, wait_until, EventListener
 
 
 class TestCommandMonitoring(unittest.TestCase):
@@ -170,12 +153,10 @@ class TestCommandMonitoring(unittest.TestCase):
         self.assertEqual('find', succeeded.command_name)
         self.assertTrue(isinstance(succeeded.request_id, int))
         self.assertEqual(cursor.address, succeeded.connection_id)
-        expected_result = {
-            'cursor': {'id': cursor_id,
-                       'ns': 'pymongo_test.test',
-                       'firstBatch': [{} for _ in range(4)]},
-            'ok': 1}
-        self.assertEqual(expected_result, succeeded.reply)
+        csr = succeeded.reply["cursor"]
+        self.assertEqual(csr["id"], cursor_id)
+        self.assertEqual(csr["ns"], "pymongo_test.test")
+        self.assertEqual(csr["firstBatch"], [{} for _ in range(4)])
 
         self.listener.results.clear()
         # Next batch. Exhausting the cursor could cause a getMore
@@ -529,7 +510,8 @@ class TestCommandMonitoring(unittest.TestCase):
             self.assertEqual(cursor.address, succeeded.connection_id)
             # There could be more than one cursor_id here depending on
             # when the thread last ran.
-            self.assertIn(cursor_id, succeeded.reply['cursorsUnknown'])
+            self.assertTrue(cursor_id in succeeded.reply['cursorsUnknown']
+                            or cursor_id in succeeded.reply['cursorsKilled'])
 
     def test_non_bulk_writes(self):
         coll = self.client.pymongo_test.test
@@ -1230,6 +1212,47 @@ class TestCommandMonitoring(unittest.TestCase):
         self.assertEqual(started.request_id, succeeded.request_id)
         self.assertEqual(started.connection_id, succeeded.connection_id)
 
+        self.listener.results.clear()
+        self.client.pymongo_test.current_op(True)
+        started = results['started'][0]
+        succeeded = results['succeeded'][0]
+        self.assertEqual(0, len(results['failed']))
+        self.assertIsInstance(started, monitoring.CommandStartedEvent)
+        expected = SON([('currentOp', 1), ('$all', True)])
+        self.assertEqual(expected, started.command)
+        self.assertEqual('admin', started.database_name)
+        self.assertEqual('currentOp', started.command_name)
+        self.assertIsInstance(started.request_id, int)
+        self.assertEqual(self.client.address, started.connection_id)
+        self.assertIsInstance(succeeded, monitoring.CommandSucceededEvent)
+        self.assertIsInstance(succeeded.duration_micros, int)
+        self.assertEqual(started.command_name, succeeded.command_name)
+        self.assertEqual(started.request_id, succeeded.request_id)
+        self.assertEqual(started.connection_id, succeeded.connection_id)
+
+        if not client_context.is_mongos:
+            self.client.fsync(lock=True)
+            self.listener.results.clear()
+            self.client.unlock()
+            # Wait for async unlock...
+            wait_until(
+                lambda: not self.client.is_locked, "unlock the database")
+            started = results['started'][0]
+            succeeded = results['succeeded'][0]
+            self.assertEqual(0, len(results['failed']))
+            self.assertIsInstance(started, monitoring.CommandStartedEvent)
+            expected = {'fsyncUnlock': 1}
+            self.assertEqual(expected, started.command)
+            self.assertEqual('admin', started.database_name)
+            self.assertEqual('fsyncUnlock', started.command_name)
+            self.assertIsInstance(started.request_id, int)
+            self.assertEqual(self.client.address, started.connection_id)
+            self.assertIsInstance(succeeded, monitoring.CommandSucceededEvent)
+            self.assertIsInstance(succeeded.duration_micros, int)
+            self.assertEqual(started.command_name, succeeded.command_name)
+            self.assertEqual(started.request_id, succeeded.request_id)
+            self.assertEqual(started.connection_id, succeeded.connection_id)
+
     def test_sensitive_commands(self):
         listeners = self.client._event_listeners
 
