@@ -1,4 +1,4 @@
-# Copyright 2009-2014 MongoDB, Inc.
+# Copyright 2009-2015 MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,19 +16,17 @@
 
 import datetime
 import pickle
-import unittest
 import sys
-sys.path[0:0] = [""]
 
-from nose.plugins.skip import SkipTest
+sys.path[0:0] = [""]
 
 from bson.errors import InvalidId
 from bson.objectid import ObjectId
-from bson.py3compat import b
+from bson.py3compat import PY3, u, _unicode
 from bson.tz_util import (FixedOffset,
                           utc)
-
-PY3 = sys.version_info[0] == 3
+from test import SkipTest, unittest
+from test.utils import oid_generated_on_client
 
 
 def oid(x):
@@ -45,73 +43,54 @@ class TestObjectId(unittest.TestCase):
         self.assertRaises(InvalidId, ObjectId, "12345678901")
         self.assertRaises(InvalidId, ObjectId, "1234567890123")
         self.assertTrue(ObjectId())
-        self.assertTrue(ObjectId(b("123456789012")))
+        self.assertTrue(ObjectId(b"123456789012"))
         a = ObjectId()
         self.assertTrue(ObjectId(a))
 
     def test_unicode(self):
         a = ObjectId()
-        self.assertEqual(a, ObjectId(unicode(a)))
+        self.assertEqual(a, ObjectId(_unicode(a)))
         self.assertEqual(ObjectId("123456789012123456789012"),
-                         ObjectId(u"123456789012123456789012"))
-        self.assertRaises(InvalidId, ObjectId, u"hello")
+                         ObjectId(u("123456789012123456789012")))
+        self.assertRaises(InvalidId, ObjectId, u("hello"))
 
     def test_from_hex(self):
         ObjectId("123456789012123456789012")
         self.assertRaises(InvalidId, ObjectId, "123456789012123456789G12")
-        self.assertRaises(InvalidId, ObjectId, u"123456789012123456789G12")
+        self.assertRaises(InvalidId, ObjectId, u("123456789012123456789G12"))
 
     def test_repr_str(self):
         self.assertEqual(repr(ObjectId("1234567890abcdef12345678")),
                          "ObjectId('1234567890abcdef12345678')")
         self.assertEqual(str(ObjectId("1234567890abcdef12345678")),
                          "1234567890abcdef12345678")
-        self.assertEqual(str(ObjectId(b("123456789012"))),
+        self.assertEqual(str(ObjectId(b"123456789012")),
                          "313233343536373839303132")
         self.assertEqual(ObjectId("1234567890abcdef12345678").binary,
-                         b('\x124Vx\x90\xab\xcd\xef\x124Vx'))
-        self.assertEqual(str(ObjectId(b('\x124Vx\x90\xab\xcd\xef\x124Vx'))),
+                         b'\x124Vx\x90\xab\xcd\xef\x124Vx')
+        self.assertEqual(str(ObjectId(b'\x124Vx\x90\xab\xcd\xef\x124Vx')),
                          "1234567890abcdef12345678")
 
     def test_equality(self):
         a = ObjectId()
         self.assertEqual(a, ObjectId(a))
-        self.assertEqual(ObjectId(b("123456789012")),
-                         ObjectId(b("123456789012")))
+        self.assertEqual(ObjectId(b"123456789012"),
+                         ObjectId(b"123456789012"))
         self.assertNotEqual(ObjectId(), ObjectId())
-        self.assertNotEqual(ObjectId(b("123456789012")), b("123456789012"))
+        self.assertNotEqual(ObjectId(b"123456789012"), b"123456789012")
 
         # Explicitly test inequality
         self.assertFalse(a != ObjectId(a))
-        self.assertFalse(ObjectId(b("123456789012")) !=
-                         ObjectId(b("123456789012")))
+        self.assertFalse(ObjectId(b"123456789012") !=
+                         ObjectId(b"123456789012"))
 
     def test_binary_str_equivalence(self):
         a = ObjectId()
         self.assertEqual(a, ObjectId(a.binary))
         self.assertEqual(a, ObjectId(str(a)))
 
-    def test_multiprocessing(self):
-        # multiprocessing on windows is weird and I don't feel like figuring it
-        # out right now. this should fix buildbot.
-        if sys.platform == "win32":
-            raise SkipTest("Can't fork on Windows")
-
-        try:
-            import multiprocessing
-        except ImportError:
-            raise SkipTest("No multiprocessing module")
-
-        pool = multiprocessing.Pool(2)
-        ids = pool.map(oid, range(20))
-        pool.close()
-        pool.join()
-
-        map = {}
-
-        for id in ids:
-            self.assertTrue(id not in map)
-            map[id] = True
+    def test_pid(self):
+        self.assertTrue(oid_generated_on_client(ObjectId()))
 
     def test_generation_time(self):
         d1 = datetime.datetime.utcnow()
@@ -151,22 +130,21 @@ class TestObjectId(unittest.TestCase):
 
         # This string was generated by pickling an ObjectId in pymongo
         # version 1.9
-        pickled_with_1_9 = b(
-            "ccopy_reg\n_reconstructor\np0\n"
-            "(cbson.objectid\nObjectId\np1\nc__builtin__\n"
-            "object\np2\nNtp3\nRp4\n"
-            "(dp5\nS'_ObjectId__id'\np6\n"
-            "S'M\\x9afV\\x13v\\xc0\\x0b\\x88\\x00\\x00\\x00'\np7\nsb.")
+        pickled_with_1_9 = (
+            b"ccopy_reg\n_reconstructor\np0\n"
+            b"(cbson.objectid\nObjectId\np1\nc__builtin__\n"
+            b"object\np2\nNtp3\nRp4\n"
+            b"(dp5\nS'_ObjectId__id'\np6\n"
+            b"S'M\\x9afV\\x13v\\xc0\\x0b\\x88\\x00\\x00\\x00'\np7\nsb.")
 
         # We also test against a hardcoded "New" pickle format so that we
         # make sure we're backward compatible with the current version in
         # the future as well.
-        pickled_with_1_10 = b(
-            "ccopy_reg\n_reconstructor\np0\n"
-            "(cbson.objectid\nObjectId\np1\nc__builtin__\n"
-            "object\np2\nNtp3\nRp4\n"
-            "S'M\\x9afV\\x13v\\xc0\\x0b\\x88\\x00\\x00\\x00'\np5\nb."
-            )
+        pickled_with_1_10 = (
+            b"ccopy_reg\n_reconstructor\np0\n"
+            b"(cbson.objectid\nObjectId\np1\nc__builtin__\n"
+            b"object\np2\nNtp3\nRp4\n"
+            b"S'M\\x9afV\\x13v\\xc0\\x0b\\x88\\x00\\x00\\x00'\np5\nb.")
 
         if PY3:
             # Have to load using 'latin-1' since these were pickled in python2.x.
@@ -189,7 +167,7 @@ class TestObjectId(unittest.TestCase):
         self.assertFalse(ObjectId.is_valid("12345678901"))
         self.assertFalse(ObjectId.is_valid("1234567890123"))
 
-        self.assertTrue(ObjectId.is_valid(b("123456789012")))
+        self.assertTrue(ObjectId.is_valid(b"123456789012"))
         self.assertTrue(ObjectId.is_valid("123456789012123456789012"))
 
 if __name__ == "__main__":
