@@ -16,6 +16,7 @@
 
 import gc
 import random
+import socket
 import sys
 import threading
 import time
@@ -241,6 +242,13 @@ class TestPooling(_TestPoolingBase):
         with cx_pool.get_socket({}):
             pass
 
+    def test_socket_closed(self):
+        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        s.connect((host, port))
+        self.assertFalse(socket_closed(s))
+        s.close()
+        self.assertTrue(socket_closed(s))
+
     def test_return_socket_after_reset(self):
         pool = self.create_pool()
         with pool.get_socket({}) as sock:
