@@ -16,6 +16,7 @@
 
 import copy
 import sys
+import warnings
 
 sys.path[0:0] = [""]
 
@@ -94,42 +95,94 @@ class TestURI(unittest.TestCase):
 
     def test_split_options(self):
         self.assertRaises(ConfigurationError, split_options, 'foo')
-        self.assertRaises(ConfigurationError, split_options, 'foo=bar')
         self.assertRaises(ConfigurationError, split_options, 'foo=bar;foo')
+        self.assertTrue(split_options('ssl=true'))
+        self.assertTrue(split_options('ssl_match_hostname=true'))
+
+        # Test Invalid URI options that should throw warnings.
+        with warnings.catch_warnings():
+            warnings.filterwarnings('error')
+            self.assertRaises(Warning, split_options,
+                              'foo=bar', warn=True)
+            self.assertRaises(Warning, split_options,
+                              'socketTimeoutMS=foo', warn=True)
+            self.assertRaises(Warning, split_options,
+                              'socketTimeoutMS=0.0', warn=True)
+            self.assertRaises(Warning, split_options,
+                              'connectTimeoutMS=foo', warn=True)
+            self.assertRaises(Warning, split_options,
+                              'connectTimeoutMS=0.0', warn=True)
+            self.assertRaises(Warning, split_options,
+                              'connectTimeoutMS=1e100000', warn=True)
+            self.assertRaises(Warning, split_options,
+                              'connectTimeoutMS=-1e100000', warn=True)
+            self.assertRaises(Warning, split_options,
+                              'ssl=foo', warn=True)
+            self.assertRaises(Warning, split_options,
+                              'ssl_match_hostname=foo', warn=True)
+
+            # On most platforms float('inf') and float('-inf') represent
+            # +/- infinity, although on Python 2.4 and 2.5 on Windows those
+            # expressions are invalid
+            if not (sys.platform == "win32" and sys.version_info <= (2, 5)):
+                self.assertRaises(Warning, split_options,
+                                  'connectTimeoutMS=inf', warn=True)
+                self.assertRaises(Warning, split_options,
+                                  'connectTimeoutMS=-inf', warn=True)
+
+            self.assertRaises(Warning, split_options, 'wtimeoutms=foo',
+                              warn=True)
+            self.assertRaises(Warning, split_options, 'wtimeoutms=5.5',
+                              warn=True)
+            self.assertRaises(Warning, split_options, 'fsync=foo',
+                              warn=True)
+            self.assertRaises(Warning, split_options, 'fsync=5.5',
+                              warn=True)
+            self.assertRaises(Warning,
+                              split_options, 'authMechanism=foo',
+                              warn=True)
+
+        # Test invalid options with warn=False.
+        self.assertRaises(ConfigurationError, split_options, 'foo=bar')
         self.assertRaises(ValueError, split_options, 'socketTimeoutMS=foo')
         self.assertRaises(ValueError, split_options, 'socketTimeoutMS=0.0')
         self.assertRaises(ValueError, split_options, 'connectTimeoutMS=foo')
         self.assertRaises(ValueError, split_options, 'connectTimeoutMS=0.0')
-        self.assertRaises(ValueError, split_options, 'connectTimeoutMS=1e100000')
-        self.assertRaises(ValueError, split_options, 'connectTimeoutMS=-1e100000')
+        self.assertRaises(ValueError, split_options,
+                          'connectTimeoutMS=1e100000')
+        self.assertRaises(ValueError, split_options,
+                          'connectTimeoutMS=-1e100000')
         self.assertRaises(ValueError, split_options, 'ssl=foo')
-        self.assertTrue(split_options('ssl=true'))
         self.assertRaises(ValueError, split_options, 'ssl_match_hostname=foo')
-        self.assertTrue(split_options('ssl_match_hostname=true'))
-
-        # On most platforms float('inf') and float('-inf') represent
-        # +/- infinity, although on Python 2.4 and 2.5 on Windows those
-        # expressions are invalid
         if not (sys.platform == "win32" and sys.version_info <= (2, 5)):
-            self.assertRaises(ValueError, split_options, 'connectTimeoutMS=inf')
-            self.assertRaises(ValueError, split_options, 'connectTimeoutMS=-inf')
+            self.assertRaises(ValueError, split_options,
+                              'connectTimeoutMS=inf')
+            self.assertRaises(ValueError, split_options,
+                              'connectTimeoutMS=-inf')
+        self.assertRaises(ValueError, split_options, 'wtimeoutms=foo')
+        self.assertRaises(ValueError, split_options, 'wtimeoutms=5.5')
+        self.assertRaises(ValueError, split_options, 'fsync=foo')
+        self.assertRaises(ValueError, split_options, 'fsync=5.5')
+        self.assertRaises(ValueError,
+                          split_options, 'authMechanism=foo')
 
+        # Test splitting options works when valid.
         self.assertTrue(split_options('socketTimeoutMS=300'))
         self.assertTrue(split_options('connectTimeoutMS=300'))
-        self.assertEqual({'sockettimeoutms': 0.3}, split_options('socketTimeoutMS=300'))
-        self.assertEqual({'sockettimeoutms': 0.0001}, split_options('socketTimeoutMS=0.1'))
-        self.assertEqual({'connecttimeoutms': 0.3}, split_options('connectTimeoutMS=300'))
-        self.assertEqual({'connecttimeoutms': 0.0001}, split_options('connectTimeoutMS=0.1'))
+        self.assertEqual({'sockettimeoutms': 0.3},
+                         split_options('socketTimeoutMS=300'))
+        self.assertEqual({'sockettimeoutms': 0.0001},
+                         split_options('socketTimeoutMS=0.1'))
+        self.assertEqual({'connecttimeoutms': 0.3},
+                         split_options('connectTimeoutMS=300'))
+        self.assertEqual({'connecttimeoutms': 0.0001},
+                         split_options('connectTimeoutMS=0.1'))
         self.assertTrue(split_options('connectTimeoutMS=300'))
         self.assertTrue(isinstance(split_options('w=5')['w'], int))
         self.assertTrue(isinstance(split_options('w=5.5')['w'], string_type))
         self.assertTrue(split_options('w=foo'))
         self.assertTrue(split_options('w=majority'))
-        self.assertRaises(ValueError, split_options, 'wtimeoutms=foo')
-        self.assertRaises(ValueError, split_options, 'wtimeoutms=5.5')
         self.assertTrue(split_options('wtimeoutms=500'))
-        self.assertRaises(ValueError, split_options, 'fsync=foo')
-        self.assertRaises(ValueError, split_options, 'fsync=5.5')
         self.assertEqual({'fsync': True}, split_options('fsync=true'))
         self.assertEqual({'fsync': False}, split_options('fsync=false'))
         self.assertEqual({'authmechanism': 'GSSAPI'},
@@ -138,9 +191,8 @@ class TestURI(unittest.TestCase):
                          split_options('authMechanism=MONGODB-CR'))
         self.assertEqual({'authmechanism': 'SCRAM-SHA-1'},
                          split_options('authMechanism=SCRAM-SHA-1'))
-        self.assertRaises(ValueError,
-                          split_options, 'authMechanism=foo')
-        self.assertEqual({'authsource': 'foobar'}, split_options('authSource=foobar'))
+        self.assertEqual({'authsource': 'foobar'},
+                         split_options('authSource=foobar'))
         self.assertEqual({'maxpoolsize': 50}, split_options('maxpoolsize=50'))
 
     def test_parse_uri(self):
@@ -204,16 +256,17 @@ class TestURI(unittest.TestCase):
                          parse_uri("mongodb://example1.com:27017,example2.com"
                                    ":27017/test.yield_historical.in"))
 
-        res = copy.deepcopy(orig)
-        res['nodelist'] = [("/tmp/mongodb-27017.sock", None)]
-        self.assertEqual(res, parse_uri("mongodb:///tmp/mongodb-27017.sock"))
+        # Test socket path without escaped characters.
+        self.assertRaises(InvalidURI, parse_uri,
+                          "mongodb:///tmp/mongodb-27017.sock")
 
+        # Test with escaped characters.
         res = copy.deepcopy(orig)
         res['nodelist'] = [("example2.com", 27017),
                            ("/tmp/mongodb-27017.sock", None)]
         self.assertEqual(res,
                          parse_uri("mongodb://example2.com,"
-                                   "/tmp/mongodb-27017.sock"))
+                                   "%2Ftmp%2Fmongodb-27017.sock"))
 
         res = copy.deepcopy(orig)
         res['nodelist'] = [("shoe.sock.pants.co.uk", 27017),
@@ -221,14 +274,14 @@ class TestURI(unittest.TestCase):
         res['database'] = "nethers_db"
         self.assertEqual(res,
                          parse_uri("mongodb://shoe.sock.pants.co.uk,"
-                                   "/tmp/mongodb-27017.sock/nethers_db"))
+                                   "%2Ftmp%2Fmongodb-27017.sock/nethers_db"))
 
         res = copy.deepcopy(orig)
         res['nodelist'] = [("/tmp/mongodb-27017.sock", None),
                            ("example2.com", 27017)]
         res.update({'database': 'test', 'collection': 'yield_historical.in'})
         self.assertEqual(res,
-                         parse_uri("mongodb:///tmp/mongodb-27017.sock,"
+                         parse_uri("mongodb://%2Ftmp%2Fmongodb-27017.sock,"
                                    "example2.com:27017"
                                    "/test.yield_historical.in"))
 
@@ -237,9 +290,9 @@ class TestURI(unittest.TestCase):
                            ("example2.com", 27017)]
         res.update({'database': 'test', 'collection': 'yield_historical.sock'})
         self.assertEqual(res,
-                         parse_uri("mongodb:///tmp/mongodb-27017.sock,"
-                                   "example2.com:27017"
-                                   "/test.yield_historical.sock"))
+                         parse_uri("mongodb://%2Ftmp%2Fmongodb-27017.sock,"
+                                   "example2.com:27017/test.yield_historical"
+                                   ".sock"))
 
         res = copy.deepcopy(orig)
         res['nodelist'] = [("example2.com", 27017)]
@@ -252,7 +305,7 @@ class TestURI(unittest.TestCase):
         res['nodelist'] = [("/tmp/mongodb-27017.sock", None)]
         res.update({'database': 'test', 'collection': 'mongodb-27017.sock'})
         self.assertEqual(res,
-                         parse_uri("mongodb:///tmp/mongodb-27017.sock"
+                         parse_uri("mongodb://%2Ftmp%2Fmongodb-27017.sock"
                                    "/test.mongodb-27017.sock"))
 
         res = copy.deepcopy(orig)
@@ -261,8 +314,8 @@ class TestURI(unittest.TestCase):
                            ("2001:0db8:85a3:0000:0000:8a2e:0370:7334", 27018),
                            ("192.168.0.212", 27019),
                            ("localhost", 27018)]
-        self.assertEqual(res, parse_uri("mongodb:///tmp/mongodb-27020.sock,"
-                                        "[::1]:27017,[2001:0db8:"
+        self.assertEqual(res, parse_uri("mongodb://%2Ftmp%2Fmongodb-27020.sock"
+                                        ",[::1]:27017,[2001:0db8:"
                                         "85a3:0000:0000:8a2e:0370:7334],"
                                         "192.168.0.212:27019,localhost",
                                         27018))
@@ -276,8 +329,8 @@ class TestURI(unittest.TestCase):
 
         res = copy.deepcopy(orig)
         res['options'] = {'readpreference': ReadPreference.SECONDARY.mode}
-        self.assertEqual(res,
-                         parse_uri("mongodb://localhost/?readPreference=secondary"))
+        self.assertEqual(res, parse_uri(
+            "mongodb://localhost/?readPreference=secondary"))
 
         # Various authentication tests
         res = copy.deepcopy(orig)
@@ -371,6 +424,12 @@ class TestURI(unittest.TestCase):
                                    "@localhost/foo?uuidrepresentation="
                                    "javaLegacy"))
 
+        with warnings.catch_warnings():
+            warnings.filterwarnings('error')
+            self.assertRaises(Warning, parse_uri,
+                              "mongodb://user%40domain.com:password"
+                              "@localhost/foo?uuidrepresentation=notAnOption",
+                              warn=True)
         self.assertRaises(ValueError, parse_uri,
                           "mongodb://user%40domain.com:password"
                           "@localhost/foo?uuidrepresentation=notAnOption")
